@@ -14,9 +14,6 @@ let test = asyncTest(function* () {
   hud.jsterm.clearOutput(true);
 
   let msg = yield execute(hud, "document.querySelector('p')");
-  let domWidget = [...msg._messageObject.widgets][0];
-  // Wait for async requests linking DOM node to inspector to complete
-  yield domWidget._linkedToInspector;
 
   is(hud.outputNode.textContent.indexOf("bug772506_content"), -1,
      "no content element found");
@@ -36,4 +33,4 @@ function execute(hud, str) {
   let deferred = promise.defer();
   hud.jsterm.execute(str, deferred.resolve);
   return deferred.promise;
-}
+}
\ No newline at end of file
