@@ -8,18 +8,21 @@
 #include "prlink.h"
 #include "plstr.h"
 #include "nsIPluginInstanceOwner.h"
-#include "nsServiceManagerUtils.h"
 #include "nsPluginsDir.h"
 #include "nsPluginHost.h"
 #include "nsIBlocklistService.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsIPlatformCharset.h"
-#include "nsICharsetConverterManager.h"
 #include "nsPluginLogging.h"
 #include "nsNPAPIPlugin.h"
+#include "nsCharSeparatedTokenizer.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/unused.h"
 #include <cctype>
+#include "mozilla/dom/EncodingUtils.h"
+#include "mozilla/dom/ContentChild.h"
 
+using mozilla::dom::EncodingUtils;
 using namespace mozilla;
 
 // These legacy flags are used in the plugin registry. The states are now
@@ -30,15 +33,68 @@ using namespace mozilla;
 // no longer used                   0x0008    // reuse only if regenerating pluginreg.dat
 #define NS_PLUGIN_FLAG_CLICKTOPLAY  0x0020    // this is a click-to-play plugin
 
-inline char* new_str(const char* str)
+static const char kPrefDefaultEnabledState[] = "plugin.default.state";
+static const char kPrefDefaultEnabledStateXpi[] = "plugin.defaultXpi.state";
+
+// check comma delimited extensions
+static bool ExtensionInList(const nsCString& aExtensionList,
+                            const nsACString& aExtension)
 {
-  if (str == nullptr)
-    return nullptr;
-  
-  char* result = new char[strlen(str) + 1];
-  if (result != nullptr)
-    return strcpy(result, str);
-  return result;
+  nsCCharSeparatedTokenizer extensions(aExtensionList, ',');
+  while (extensions.hasMoreTokens()) {
+    const nsCSubstring& extension = extensions.nextToken();
+    if (extension.Equals(aExtension, nsCaseInsensitiveCStringComparator())) {
+      return true;
+    }
+  }
+  return false;
+}
+
+// Search for an extension in an extensions array, and return its
+// matching mime type
+static bool SearchExtensions(const nsTArray<nsCString> & aExtensions,
+                             const nsTArray<nsCString> & aMimeTypes,
+                             const nsACString & aFindExtension,
+                             nsACString & aMatchingType)
+{
+  uint32_t mimes = aMimeTypes.Length();
+  MOZ_ASSERT(mimes == aExtensions.Length(),
+             "These arrays should have matching elements");
+
+  aMatchingType.Truncate();
+
+  for (uint32_t i = 0; i < mimes; i++) {
+    if (ExtensionInList(aExtensions[i], aFindExtension)) {
+      aMatchingType = aMimeTypes[i];
+      return true;
+    }
+  }
+
+  return false;
+}
+
+static nsCString
+MakeNiceFileName(const nsCString & aFileName)
+{
+  nsCString niceName = aFileName;
+  int32_t niceNameLength = aFileName.RFind(".");
+  NS_ASSERTION(niceNameLength != kNotFound, "aFileName doesn't have a '.'?");
+  while (niceNameLength > 0) {
+    char chr = aFileName[niceNameLength - 1];
+    if (!std::isalpha(chr))
+      niceNameLength--;
+    else
+      break;
+  }
+
+  // If it turns out that niceNameLength <= 0, we'll fall back and use the
+  // entire aFileName (which we've already taken care of, a few lines back).
+  if (niceNameLength > 0) {
+    niceName.Truncate(niceNameLength);
+  }
+
+  ToLowerCase(niceName);
+  return niceName;
 }
 
 static nsCString
@@ -54,6 +110,29 @@ MakePrefNameForPlugin(const char* const subname, nsPluginTag* aTag)
   return pref;
 }
 
+static nsresult
+CStringArrayToXPCArray(nsTArray<nsCString> & aArray,
+                       uint32_t* aCount,
+                       char16_t*** aResults)
+{
+  uint32_t count = aArray.Length();
+  if (!count) {
+    *aResults = nullptr;
+    *aCount = 0;
+    return NS_OK;
+  }
+
+  *aResults =
+    static_cast<char16_t**>(moz_xmalloc(count * sizeof(**aResults)));
+  *aCount = count;
+
+  for (uint32_t i = 0; i < count; i++) {
+    (*aResults)[i] = ToNewUnicode(NS_ConvertUTF8toUTF16(aArray[i]));
+  }
+
+  return NS_OK;
+}
+
 static nsCString
 GetStatePrefNameForPlugin(nsPluginTag* aTag)
 {
@@ -62,8 +141,15 @@ GetStatePrefNameForPlugin(nsPluginTag* aTag)
 
 /* nsPluginTag */
 
-nsPluginTag::nsPluginTag(nsPluginInfo* aPluginInfo)
-  : mName(aPluginInfo->fName),
+uint32_t nsPluginTag::sNextId;
+
+nsPluginTag::nsPluginTag(nsPluginInfo* aPluginInfo,
+                         int64_t aLastModifiedTime,
+                         bool fromExtension)
+  : mId(sNextId++),
+    mContentProcessRunningCount(0),
+    mHadLocalInstance(false),
+    mName(aPluginInfo->fName),
     mDescription(aPluginInfo->fDescription),
     mLibrary(nullptr),
     mIsJavaPlugin(false),
@@ -71,10 +157,10 @@ nsPluginTag::nsPluginTag(nsPluginInfo* aPluginInfo)
     mFileName(aPluginInfo->fFileName),
     mFullPath(aPluginInfo->fFullPath),
     mVersion(aPluginInfo->fVersion),
-    mLastModifiedTime(0),
-    mNiceFileName(),
+    mLastModifiedTime(aLastModifiedTime),
     mCachedBlocklistState(nsIBlocklistService::STATE_NOT_BLOCKED),
-    mCachedBlocklistStateValid(false)
+    mCachedBlocklistStateValid(false),
+    mIsFromExtension(fromExtension)
 {
   InitMime(aPluginInfo->fMimeTypeArray,
            aPluginInfo->fMimeDescriptionArray,
@@ -94,19 +180,23 @@ nsPluginTag::nsPluginTag(const char* aName,
                          const char* const* aExtensions,
                          int32_t aVariants,
                          int64_t aLastModifiedTime,
+                         bool fromExtension,
                          bool aArgsAreUTF8)
-  : mName(aName),
+  : mId(sNextId++),
+    mContentProcessRunningCount(0),
+    mName(aName),
     mDescription(aDescription),
     mLibrary(nullptr),
     mIsJavaPlugin(false),
     mIsFlashPlugin(false),
+    mSupportsAsyncInit(false),
     mFileName(aFileName),
     mFullPath(aFullPath),
     mVersion(aVersion),
     mLastModifiedTime(aLastModifiedTime),
-    mNiceFileName(),
     mCachedBlocklistState(nsIBlocklistService::STATE_NOT_BLOCKED),
-    mCachedBlocklistStateValid(false)
+    mCachedBlocklistStateValid(false),
+    mIsFromExtension(fromExtension)
 {
   InitMime(aMimeTypes, aMimeDescriptions, aExtensions,
            static_cast<uint32_t>(aVariants));
@@ -115,12 +205,45 @@ nsPluginTag::nsPluginTag(const char* aName,
   FixupVersion();
 }
 
+nsPluginTag::nsPluginTag(uint32_t aId,
+                         const char* aName,
+                         const char* aDescription,
+                         const char* aFileName,
+                         const char* aFullPath,
+                         const char* aVersion,
+                         nsTArray<nsCString> aMimeTypes,
+                         nsTArray<nsCString> aMimeDescriptions,
+                         nsTArray<nsCString> aExtensions,
+                         bool aIsJavaPlugin,
+                         bool aIsFlashPlugin,
+                         int64_t aLastModifiedTime,
+                         bool aFromExtension)
+  : mId(aId),
+    mContentProcessRunningCount(0),
+    mName(aName),
+    mDescription(aDescription),
+    mMimeTypes(aMimeTypes),
+    mMimeDescriptions(aMimeDescriptions),
+    mExtensions(aExtensions),
+    mLibrary(nullptr),
+    mIsJavaPlugin(aIsJavaPlugin),
+    mIsFlashPlugin(aIsFlashPlugin),
+    mFileName(aFileName),
+    mVersion(aVersion),
+    mLastModifiedTime(aLastModifiedTime),
+    mNiceFileName(),
+    mCachedBlocklistState(nsIBlocklistService::STATE_NOT_BLOCKED),
+    mCachedBlocklistStateValid(false),
+    mIsFromExtension(aFromExtension)
+{
+}
+
 nsPluginTag::~nsPluginTag()
 {
   NS_ASSERTION(!mNext, "Risk of exhausting the stack space, bug 486349");
 }
 
-NS_IMPL_ISUPPORTS1(nsPluginTag, nsIPluginTag)
+NS_IMPL_ISUPPORTS(nsPluginTag, nsIPluginTag)
 
 void nsPluginTag::InitMime(const char* const* aMimeTypes,
                            const char* const* aMimeDescriptions,
@@ -147,10 +270,26 @@ void nsPluginTag::InitMime(const char* const* aMimeTypes,
     }
 
     // Look for certain special plugins.
-    if (nsPluginHost::IsJavaMIMEType(mimeType.get())) {
-      mIsJavaPlugin = true;
-    } else if (mimeType.EqualsLiteral("application/x-shockwave-flash")) {
-      mIsFlashPlugin = true;
+    switch (nsPluginHost::GetSpecialType(mimeType)) {
+      case nsPluginHost::eSpecialType_Java:
+        mIsJavaPlugin = true;
+        mSupportsAsyncInit = true;
+        break;
+      case nsPluginHost::eSpecialType_Flash:
+        mIsFlashPlugin = true;
+        mSupportsAsyncInit = true;
+        break;
+      case nsPluginHost::eSpecialType_Silverlight:
+      case nsPluginHost::eSpecialType_Unity:
+        mSupportsAsyncInit = true;
+        break;
+      case nsPluginHost::eSpecialType_None:
+      default:
+#ifndef RELEASE_BUILD
+        // Allow async init for all plugins on Nightly and Aurora
+        mSupportsAsyncInit = true;
+#endif
+        break;
     }
 
     // Fill in our MIME type array.
@@ -205,14 +344,14 @@ static nsresult ConvertToUTF8(nsIUnicodeDecoder *aUnicodeDecoder,
   nsresult rv = aUnicodeDecoder->GetMaxLength(aString.get(), numberOfBytes,
                                               &outUnicodeLen);
   NS_ENSURE_SUCCESS(rv, rv);
-  if (!buffer.SetLength(outUnicodeLen, fallible_t()))
+  if (!buffer.SetLength(outUnicodeLen, fallible))
     return NS_ERROR_OUT_OF_MEMORY;
   rv = aUnicodeDecoder->Convert(aString.get(), &numberOfBytes,
                                 buffer.BeginWriting(), &outUnicodeLen);
   NS_ENSURE_SUCCESS(rv, rv);
   buffer.SetLength(outUnicodeLen);
   CopyUTF16toUTF8(buffer, aString);
-  
+
   return NS_OK;
 }
 #endif
@@ -223,22 +362,17 @@ nsresult nsPluginTag::EnsureMembersAreUTF8()
   return NS_OK;
 #else
   nsresult rv;
-  
+
   nsCOMPtr<nsIPlatformCharset> pcs =
   do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIUnicodeDecoder> decoder;
-  nsCOMPtr<nsICharsetConverterManager> ccm =
-  do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-  
+
   nsAutoCString charset;
   rv = pcs->GetCharset(kPlatformCharsetSel_FileName, charset);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!charset.LowerCaseEqualsLiteral("utf-8")) {
-    rv = ccm->GetUnicodeDecoderRaw(charset.get(), getter_AddRefs(decoder));
-    NS_ENSURE_SUCCESS(rv, rv);
-    
+    decoder = EncodingUtils::DecoderForEncoding(charset);
     ConvertToUTF8(decoder, mFileName);
     ConvertToUTF8(decoder, mFullPath);
   }
@@ -249,9 +383,7 @@ nsresult nsPluginTag::EnsureMembersAreUTF8()
   rv = pcs->GetCharset(kPlatformCharsetSel_PlainTextInFile, charset);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!charset.LowerCaseEqualsLiteral("utf-8")) {
-    rv = ccm->GetUnicodeDecoderRaw(charset.get(), getter_AddRefs(decoder));
-    NS_ENSURE_SUCCESS(rv, rv);
-    
+    decoder = EncodingUtils::DecoderForEncoding(charset);
     ConvertToUTF8(decoder, mName);
     ConvertToUTF8(decoder, mDescription);
     for (uint32_t i = 0; i < mMimeDescriptions.Length(); ++i) {
@@ -312,6 +444,13 @@ nsPluginTag::IsActive()
   return IsEnabled() && !IsBlocklisted();
 }
 
+NS_IMETHODIMP
+nsPluginTag::GetActive(bool *aResult)
+{
+  *aResult = IsActive();
+  return NS_OK;
+}
+
 bool
 nsPluginTag::IsEnabled()
 {
@@ -329,7 +468,9 @@ nsPluginTag::GetDisabled(bool* aDisabled)
 bool
 nsPluginTag::IsBlocklisted()
 {
-  return GetBlocklistState() == nsIBlocklistService::STATE_BLOCKED;
+  uint32_t blocklistState;
+  nsresult rv = GetBlocklistState(&blocklistState);
+  return NS_FAILED(rv) || blocklistState == nsIBlocklistService::STATE_BLOCKED;
 }
 
 NS_IMETHODIMP
@@ -339,6 +480,22 @@ nsPluginTag::GetBlocklisted(bool* aBlocklisted)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPluginTag::GetIsEnabledStateLocked(bool* aIsEnabledStateLocked)
+{
+  *aIsEnabledStateLocked = false;
+  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
+
+  if (NS_WARN_IF(!prefs)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  unused << prefs->PrefIsLocked(GetStatePrefNameForPlugin(this).get(),
+                                aIsEnabledStateLocked);
+
+  return NS_OK;
+}
+
 bool
 nsPluginTag::IsClicktoplay()
 {
@@ -365,8 +522,10 @@ nsPluginTag::GetEnabledState(uint32_t *aEnabledState) {
     return rv;
   }
 
-  enabledState = Preferences::GetInt("plugin.default.state",
-                                     nsIPluginTag::STATE_ENABLED);
+  const char* const pref = mIsFromExtension ? kPrefDefaultEnabledStateXpi
+                                            : kPrefDefaultEnabledState;
+
+  enabledState = Preferences::GetInt(pref, nsIPluginTag::STATE_ENABLED);
   if (enabledState >= nsIPluginTag::STATE_DISABLED &&
       enabledState <= nsIPluginTag::STATE_ENABLED) {
     *aEnabledState = (uint32_t)enabledState;
@@ -409,54 +568,21 @@ nsPluginTag::SetPluginState(PluginState state)
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetMimeTypes(uint32_t* aCount, PRUnichar*** aResults)
+nsPluginTag::GetMimeTypes(uint32_t* aCount, char16_t*** aResults)
 {
-  uint32_t count = mMimeTypes.Length();
-  *aResults = static_cast<PRUnichar**>
-                         (nsMemory::Alloc(count * sizeof(**aResults)));
-  if (!*aResults)
-    return NS_ERROR_OUT_OF_MEMORY;
-  *aCount = count;
-
-  for (uint32_t i = 0; i < count; i++) {
-    (*aResults)[i] = ToNewUnicode(mMimeTypes[i]);
-  }
-
-  return NS_OK;
+  return CStringArrayToXPCArray(mMimeTypes, aCount, aResults);
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetMimeDescriptions(uint32_t* aCount, PRUnichar*** aResults)
+nsPluginTag::GetMimeDescriptions(uint32_t* aCount, char16_t*** aResults)
 {
-  uint32_t count = mMimeDescriptions.Length();
-  *aResults = static_cast<PRUnichar**>
-                         (nsMemory::Alloc(count * sizeof(**aResults)));
-  if (!*aResults)
-    return NS_ERROR_OUT_OF_MEMORY;
-  *aCount = count;
-
-  for (uint32_t i = 0; i < count; i++) {
-    (*aResults)[i] = ToNewUnicode(mMimeDescriptions[i]);
-  }
-
-  return NS_OK;
+  return CStringArrayToXPCArray(mMimeDescriptions, aCount, aResults);
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetExtensions(uint32_t* aCount, PRUnichar*** aResults)
+nsPluginTag::GetExtensions(uint32_t* aCount, char16_t*** aResults)
 {
-  uint32_t count = mExtensions.Length();
-  *aResults = static_cast<PRUnichar**>
-                         (nsMemory::Alloc(count * sizeof(**aResults)));
-  if (!*aResults)
-    return NS_ERROR_OUT_OF_MEMORY;
-  *aCount = count;
-
-  for (uint32_t i = 0; i < count; i++) {
-    (*aResults)[i] = ToNewUnicode(mExtensions[i]);
-  }
-
-  return NS_OK;
+  return CStringArrayToXPCArray(mExtensions, aCount, aResults);
 }
 
 bool
@@ -482,11 +608,10 @@ void nsPluginTag::TryUnloadPlugin(bool inShutdown)
 {
   // We never want to send NPP_Shutdown to an in-process plugin unless
   // this process is shutting down.
-  if (mLibrary && !inShutdown) {
+  if (!mPlugin) {
     return;
   }
-
-  if (mPlugin) {
+  if (inShutdown || mPlugin->GetLibrary()->IsOOP()) {
     mPlugin->Shutdown();
     mPlugin = nullptr;
   }
@@ -498,36 +623,26 @@ nsCString nsPluginTag::GetNiceFileName() {
   }
 
   if (mIsFlashPlugin) {
-    mNiceFileName.Assign(NS_LITERAL_CSTRING("flash"));
+    mNiceFileName.AssignLiteral("flash");
     return mNiceFileName;
   }
 
   if (mIsJavaPlugin) {
-    mNiceFileName.Assign(NS_LITERAL_CSTRING("java"));
+    mNiceFileName.AssignLiteral("java");
     return mNiceFileName;
   }
 
-  mNiceFileName.Assign(mFileName);
-  int32_t niceNameLength = mFileName.RFind(".");
-  NS_ASSERTION(niceNameLength != kNotFound, "mFileName doesn't have a '.'?");
-  while (niceNameLength > 0) {
-    char chr = mFileName[niceNameLength - 1];
-    if (!std::isalpha(chr))
-      niceNameLength--;
-    else
-      break;
-  }
-
-  // If it turns out that niceNameLength <= 0, we'll fall back and use the
-  // entire mFileName (which we've already taken care of, a few lines back)
-  if (niceNameLength > 0) {
-    mNiceFileName.Truncate(niceNameLength);
-  }
-
-  ToLowerCase(mNiceFileName);
+  mNiceFileName = MakeNiceFileName(mFileName);
   return mNiceFileName;
 }
 
+NS_IMETHODIMP
+nsPluginTag::GetNiceName(nsACString & aResult)
+{
+  aResult = GetNiceFileName();
+  return NS_OK;
+}
+
 void nsPluginTag::ImportFlagsToPrefs(uint32_t flags)
 {
   if (!(flags & NS_PLUGIN_FLAG_ENABLED)) {
@@ -535,30 +650,61 @@ void nsPluginTag::ImportFlagsToPrefs(uint32_t flags)
   }
 }
 
-uint32_t
-nsPluginTag::GetBlocklistState()
+NS_IMETHODIMP
+nsPluginTag::GetBlocklistState(uint32_t *aResult)
 {
+#if defined(MOZ_WIDGET_ANDROID)
+  *aResult = nsIBlocklistService::STATE_NOT_BLOCKED;
+  return NS_OK;
+#else
   if (mCachedBlocklistStateValid) {
-    return mCachedBlocklistState;
+    *aResult = mCachedBlocklistState;
+    return NS_OK;
   }
 
-  nsCOMPtr<nsIBlocklistService> blocklist = do_GetService("@mozilla.org/extensions/blocklist;1");
-  if (!blocklist) {
-    return nsIBlocklistService::STATE_NOT_BLOCKED;
+  if (XRE_GetProcessType() != GeckoProcessType_Default) {
+    *aResult = nsIBlocklistService::STATE_BLOCKED;
+    dom::ContentChild* cp = dom::ContentChild::GetSingleton();
+    if (!cp->SendGetBlocklistState(mId, aResult)) {
+      return NS_OK;
+    }
+  } else {
+    nsCOMPtr<nsIBlocklistService> blocklist =
+      do_GetService("@mozilla.org/extensions/blocklist;1");
+
+    if (!blocklist) {
+      *aResult = nsIBlocklistService::STATE_NOT_BLOCKED;
+      return NS_OK;
+    }
+
+    // The EmptyString()s are so we use the currently running application
+    // and toolkit versions
+    if (NS_FAILED(blocklist->GetPluginBlocklistState(this, EmptyString(),
+                                                     EmptyString(), aResult))) {
+      *aResult = nsIBlocklistService::STATE_NOT_BLOCKED;
+      return NS_OK;
+    }
   }
 
-  // The EmptyString()s are so we use the currently running application
-  // and toolkit versions
-  uint32_t state;
-  if (NS_FAILED(blocklist->GetPluginBlocklistState(this, EmptyString(),
-                                                   EmptyString(), &state))) {
-    return nsIBlocklistService::STATE_NOT_BLOCKED;
-  }
-
-  MOZ_ASSERT(state <= UINT16_MAX);
-  mCachedBlocklistState = (uint16_t) state;
+  MOZ_ASSERT(*aResult <= UINT16_MAX);
+  mCachedBlocklistState = (uint16_t) *aResult;
   mCachedBlocklistStateValid = true;
-  return state;
+  return NS_OK;
+#endif // defined(MOZ_WIDGET_ANDROID)
+}
+
+bool
+nsPluginTag::HasMimeType(const nsACString & aMimeType) const
+{
+  return mMimeTypes.Contains(aMimeType,
+                             nsCaseInsensitiveCStringArrayComparator());
+}
+
+bool
+nsPluginTag::HasExtension(const nsACString & aExtension,
+                          nsACString & aMatchingType) const
+{
+  return SearchExtensions(mExtensions, mMimeTypes, aExtension, aMatchingType);
 }
 
 void
@@ -566,3 +712,16 @@ nsPluginTag::InvalidateBlocklistState()
 {
   mCachedBlocklistStateValid = false;
 }
+
+NS_IMETHODIMP
+nsPluginTag::GetLastModifiedTime(PRTime* aLastModifiedTime)
+{
+  MOZ_ASSERT(aLastModifiedTime);
+  *aLastModifiedTime = mLastModifiedTime;
+  return NS_OK;
+}
+
+bool nsPluginTag::IsFromExtension() const
+{
+  return mIsFromExtension;
+}
