@@ -4,7 +4,9 @@
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "TextureHostBasic.h"
+#ifdef XP_MACOSX
 #include "MacIOSurfaceTextureHostBasic.h"
+#endif
 
 using namespace mozilla::gl;
 using namespace mozilla::gfx;
@@ -13,28 +15,18 @@ namespace mozilla {
 namespace layers {
 
 TemporaryRef<TextureHost>
-CreateTextureHostBasic(uint64_t aID,
-                       const SurfaceDescriptor& aDesc,
+CreateTextureHostBasic(const SurfaceDescriptor& aDesc,
                        ISurfaceAllocator* aDeallocator,
                        TextureFlags aFlags)
 {
-  RefPtr<TextureHost> result;
-  switch (aDesc.type()) {
 #ifdef XP_MACOSX
-    case SurfaceDescriptor::TSurfaceDescriptorMacIOSurface: {
-      const SurfaceDescriptorMacIOSurface& desc =
-        aDesc.get_SurfaceDescriptorMacIOSurface();
-      result = new MacIOSurfaceTextureHostBasic(aID, aFlags, desc);
-      break;
-    }
-#endif
-    default: {
-      result = CreateBackendIndependentTextureHost(aID, aDesc, aDeallocator, aFlags);
-      break;
-    }
+  if (aDesc.type() == SurfaceDescriptor::TSurfaceDescriptorMacIOSurface) {
+    const SurfaceDescriptorMacIOSurface& desc =
+      aDesc.get_SurfaceDescriptorMacIOSurface();
+    return MakeAndAddRef<MacIOSurfaceTextureHostBasic>(aFlags, desc);
   }
-
-  return result;
+#endif
+  return CreateBackendIndependentTextureHost(aDesc, aDeallocator, aFlags);
 }
 
 } // namespace layers
