@@ -27,23 +27,23 @@ public:
     }
 
     virtual already_AddRefed<gfxASurface>
-      CreateOffscreenSurface(const IntSize& size,
-                             gfxContentType contentType) MOZ_OVERRIDE;
+      CreateOffscreenSurface(const IntSize& aSize,
+                             gfxImageFormat aFormat) override;
 
     virtual mozilla::TemporaryRef<mozilla::gfx::ScaledFont>
-      GetScaledFontForFont(mozilla::gfx::DrawTarget* aTarget, gfxFont *aFont) MOZ_OVERRIDE;
+      GetScaledFontForFont(mozilla::gfx::DrawTarget* aTarget, gfxFont *aFont) override;
 
     virtual nsresult GetFontList(nsIAtom *aLangGroup,
                                  const nsACString& aGenericFamily,
-                                 nsTArray<nsString>& aListOfFonts) MOZ_OVERRIDE;
+                                 nsTArray<nsString>& aListOfFonts) override;
 
-    virtual nsresult UpdateFontList() MOZ_OVERRIDE;
+    virtual nsresult UpdateFontList() override;
 
-    virtual nsresult GetStandardFamilyName(const nsAString& aFontName, nsAString& aFamilyName) MOZ_OVERRIDE;
+    virtual nsresult GetStandardFamilyName(const nsAString& aFontName, nsAString& aFamilyName) override;
 
     virtual gfxFontGroup *CreateFontGroup(const mozilla::FontFamilyList& aFontFamilyList,
                                           const gfxFontStyle *aStyle,
-                                          gfxUserFontSet* aUserFontSet) MOZ_OVERRIDE;
+                                          gfxUserFontSet* aUserFontSet) override;
 
     /**
      * Look up a local platform font using the full font face name (needed to
@@ -52,7 +52,7 @@ public:
     virtual gfxFontEntry* LookupLocalFont(const nsAString& aFontName,
                                           uint16_t aWeight,
                                           int16_t aStretch,
-                                          bool aItalic) MOZ_OVERRIDE;
+                                          bool aItalic) override;
 
     /**
      * Activate a platform font (needed to support @font-face src url() )
@@ -63,30 +63,30 @@ public:
                                            int16_t aStretch,
                                            bool aItalic,
                                            const uint8_t* aFontData,
-                                           uint32_t aLength) MOZ_OVERRIDE;
+                                           uint32_t aLength) override;
 
     /**
      * Check whether format is supported on a platform or not (if unclear,
      * returns true).
      */
     virtual bool IsFontFormatSupported(nsIURI *aFontURI,
-                                       uint32_t aFormatFlags) MOZ_OVERRIDE;
+                                       uint32_t aFormatFlags) override;
 
     static int32_t GetDPI();
 
-    virtual gfxImageFormat GetOffscreenFormat() MOZ_OVERRIDE;
+    virtual gfxImageFormat GetOffscreenFormat() override;
 #ifdef MOZ_X11
     static Display* GetXDisplay(QWindow* aWindow = 0);
     static Screen* GetXScreen(QWindow* aWindow = 0);
 #endif
 
-    virtual int GetScreenDepth() const MOZ_OVERRIDE;
+    virtual int GetScreenDepth() const override;
 
 protected:
     static gfxFontconfigUtils *sFontconfigUtils;
 
 private:
-    virtual void GetPlatformCMSOutputProfile(void *&mem, size_t &size) MOZ_OVERRIDE;
+    virtual void GetPlatformCMSOutputProfile(void *&mem, size_t &size) override;
 
     int mScreenDepth;
 };
