@@ -19,13 +19,12 @@
 #include "vm/NativeObject-inl.h"
 
 using namespace js;
-using namespace js::types;
 
 using mozilla::ArrayLength;
 using mozilla::Maybe;
 
 bool
-js::CreateRegExpMatchResult(JSContext *cx, HandleString input, const MatchPairs &matches,
+js::CreateRegExpMatchResult(JSContext* cx, HandleString input, const MatchPairs& matches,
                             MutableHandleValue rval)
 {
     MOZ_ASSERT(input);
@@ -41,7 +40,7 @@ js::CreateRegExpMatchResult(JSContext *cx, HandleString input, const MatchPairs
      */
 
     /* Get the templateObject that defines the shape and type of the output object */
-    JSObject *templateObject = cx->compartment()->regExps.getOrCreateMatchResultTemplateObject(cx);
+    JSObject* templateObject = cx->compartment()->regExps.getOrCreateMatchResultTemplateObject(cx);
     if (!templateObject)
         return false;
 
@@ -54,14 +53,14 @@ js::CreateRegExpMatchResult(JSContext *cx, HandleString input, const MatchPairs
 
     /* Store a Value for each pair. */
     for (size_t i = 0; i < numPairs; i++) {
-        const MatchPair &pair = matches[i];
+        const MatchPair& pair = matches[i];
 
         if (pair.isUndefined()) {
             MOZ_ASSERT(i != 0); /* Since we had a match, first pair must be present. */
             arr->setDenseInitializedLength(i + 1);
             arr->initDenseElement(i, UndefinedValue());
         } else {
-            JSLinearString *str = NewDependentString(cx, input, pair.start, pair.length());
+            JSLinearString* str = NewDependentString(cx, input, pair.start, pair.length());
             if (!str)
                 return false;
             arr->setDenseInitializedLength(i + 1);
@@ -78,11 +77,11 @@ js::CreateRegExpMatchResult(JSContext *cx, HandleString input, const MatchPairs
 #ifdef DEBUG
     RootedValue test(cx);
     RootedId id(cx, NameToId(cx->names().index));
-    if (!baseops::GetProperty(cx, arr, id, &test))
+    if (!NativeGetProperty(cx, arr, id, &test))
         return false;
     MOZ_ASSERT(test == arr->getSlot(0));
     id = NameToId(cx->names().input);
-    if (!baseops::GetProperty(cx, arr, id, &test))
+    if (!NativeGetProperty(cx, arr, id, &test))
         return false;
     MOZ_ASSERT(test == arr->getSlot(1));
 #endif
@@ -92,8 +91,8 @@ js::CreateRegExpMatchResult(JSContext *cx, HandleString input, const MatchPairs
 }
 
 static RegExpRunStatus
-ExecuteRegExpImpl(JSContext *cx, RegExpStatics *res, RegExpShared &re, HandleLinearString input,
-                  size_t searchIndex, MatchPairs *matches)
+ExecuteRegExpImpl(JSContext* cx, RegExpStatics* res, RegExpShared& re, HandleLinearString input,
+                  size_t searchIndex, MatchPairs* matches)
 {
     RegExpRunStatus status = re.execute(cx, input, searchIndex, matches);
     if (status == RegExpRunStatus_Success && res) {
@@ -109,8 +108,8 @@ ExecuteRegExpImpl(JSContext *cx, RegExpStatics *res, RegExpShared &re, HandleLin
 
 /* Legacy ExecuteRegExp behavior is baked into the JSAPI. */
 bool
-js::ExecuteRegExpLegacy(JSContext *cx, RegExpStatics *res, RegExpObject &reobj,
-                        HandleLinearString input, size_t *lastIndex, bool test,
+js::ExecuteRegExpLegacy(JSContext* cx, RegExpStatics* res, RegExpObject& reobj,
+                        HandleLinearString input, size_t* lastIndex, bool test,
                         MutableHandleValue rval)
 {
     RegExpGuard shared(cx);
@@ -140,223 +139,376 @@ js::ExecuteRegExpLegacy(JSContext *cx, RegExpStatics *res, RegExpObject &reobj,
     return CreateRegExpMatchResult(cx, input, matches, rval);
 }
 
-/* Note: returns the original if no escaping need be performed. */
-template <typename CharT>
-static bool
-EscapeNakedForwardSlashes(StringBuffer &sb, const CharT *oldChars, size_t oldLen)
+/* ES6 draft rc4 21.2.3.2.2. */
+bool
+RegExpInitialize(JSContext* cx, RegExpObjectBuilder& builder,
+                 HandleValue patternValue, HandleValue flagsValue,
+                 RegExpStaticsUse staticsUse, MutableHandleObject result)
 {
-    for (const CharT *it = oldChars; it < oldChars + oldLen; ++it) {
-        if (*it == '/' && (it == oldChars || it[-1] != '\\')) {
-            /* There's a forward slash that needs escaping. */
-            if (sb.empty()) {
-                /* This is the first one we've seen, copy everything up to this point. */
-                if (mozilla::IsSame<CharT, char16_t>::value && !sb.ensureTwoByteChars())
-                    return false;
-
-                if (!sb.reserve(oldLen + 1))
-                    return false;
-
-                sb.infallibleAppend(oldChars, size_t(it - oldChars));
-            }
-            if (!sb.append('\\'))
-                return false;
-        }
-
-        if (!sb.empty() && !sb.append(*it))
-            return false;
-    }
-
-    return true;
-}
-
-static JSAtom *
-EscapeNakedForwardSlashes(JSContext *cx, JSAtom *unescaped)
-{
-    /* We may never need to use |sb|. Start using it lazily. */
-    StringBuffer sb(cx);
-
-    if (unescaped->hasLatin1Chars()) {
-        JS::AutoCheckCannotGC nogc;
-        if (!EscapeNakedForwardSlashes(sb, unescaped->latin1Chars(nogc), unescaped->length()))
-            return nullptr;
+    RootedAtom pattern(cx);
+    if (patternValue.isUndefined()) {
+        /* Step 1. */
+        pattern = cx->runtime()->emptyString;
     } else {
-        JS::AutoCheckCannotGC nogc;
-        if (!EscapeNakedForwardSlashes(sb, unescaped->twoByteChars(nogc), unescaped->length()))
-            return nullptr;
-    }
-
-    return sb.empty() ? unescaped : sb.finishAtom();
-}
-
-/*
- * Compile a new |RegExpShared| for the |RegExpObject|.
- *
- * Per ECMAv5 15.10.4.1, we act on combinations of (pattern, flags) as
- * arguments:
- *
- *  RegExp, undefined => flags := pattern.flags
- *  RegExp, _ => throw TypeError
- *  _ => pattern := ToString(pattern) if defined(pattern) else ''
- *       flags := ToString(flags) if defined(flags) else ''
- */
-static bool
-CompileRegExpObject(JSContext *cx, RegExpObjectBuilder &builder, CallArgs args)
-{
-    if (args.length() == 0) {
-        RegExpStatics *res = cx->global()->getRegExpStatics(cx);
-        if (!res)
-            return false;
-        Rooted<JSAtom*> empty(cx, cx->runtime()->emptyString);
-        RegExpObject *reobj = builder.build(empty, res->getFlags());
-        if (!reobj)
-            return false;
-        args.rval().setObject(*reobj);
-        return true;
-    }
-
-    RootedValue sourceValue(cx, args[0]);
-
-    /*
-     * If we get passed in an object whose internal [[Class]] property is
-     * "RegExp", return a new object with the same source/flags.
-     */
-    if (IsObjectWithClass(sourceValue, ESClass_RegExp, cx)) {
-        /*
-         * Beware, sourceObj may be a (transparent) proxy to a RegExp, so only
-         * use generic (proxyable) operations on sourceObj that do not assume
-         * sourceObj.is<RegExpObject>().
-         */
-        RootedObject sourceObj(cx, &sourceValue.toObject());
-
-        if (args.hasDefined(1)) {
-            JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_NEWREGEXP_FLAGGED);
-            return false;
-        }
-
-        /*
-         * Only extract the 'flags' out of sourceObj; do not reuse the
-         * RegExpShared since it may be from a different compartment.
-         */
-        RegExpFlag flags;
-        {
-            RegExpGuard g(cx);
-            if (!RegExpToShared(cx, sourceObj, &g))
-                return false;
-
-            flags = g->getFlags();
-        }
-
-        /*
-         * 'toSource' is a permanent read-only property, so this is equivalent
-         * to executing RegExpObject::getSource on the unwrapped object.
-         */
-        RootedValue v(cx);
-        if (!JSObject::getProperty(cx, sourceObj, sourceObj, cx->names().source, &v))
-            return false;
-
-        // For proxies like CPOWs, we can't assume the result of a property get
-        // for 'source' is atomized.
-        Rooted<JSAtom*> sourceAtom(cx, AtomizeString(cx, v.toString()));
-        RegExpObject *reobj = builder.build(sourceAtom, flags);
-        if (!reobj)
-            return false;
-
-        args.rval().setObject(*reobj);
-        return true;
-    }
-
-    RootedAtom source(cx);
-    if (sourceValue.isUndefined()) {
-        source = cx->runtime()->emptyString;
-    } else {
-        /* Coerce to string and compile. */
-        source = ToAtom<CanGC>(cx, sourceValue);
-        if (!source)
+        /* Steps 2-3. */
+        pattern = ToAtom<CanGC>(cx, patternValue);
+        if (!pattern)
             return false;
     }
 
+    /* Step 4. */
     RegExpFlag flags = RegExpFlag(0);
-    if (args.hasDefined(1)) {
-        RootedString flagStr(cx, ToString<CanGC>(cx, args[1]));
+    if (!flagsValue.isUndefined()) {
+        /* Steps 5-6. */
+        RootedString flagStr(cx, ToString<CanGC>(cx, flagsValue));
         if (!flagStr)
             return false;
-        args[1].setString(flagStr);
+        /* Step 7. */
         if (!ParseRegExpFlags(cx, flagStr, &flags))
             return false;
     }
 
-    RootedAtom escapedSourceStr(cx, EscapeNakedForwardSlashes(cx, source));
-    if (!escapedSourceStr)
-        return false;
-
+    /* Steps 9-10. */
     CompileOptions options(cx);
     frontend::TokenStream dummyTokenStream(cx, options, nullptr, 0, nullptr);
-
-    if (!irregexp::ParsePatternSyntax(dummyTokenStream, cx->tempLifoAlloc(), escapedSourceStr))
+    if (!irregexp::ParsePatternSyntax(dummyTokenStream, cx->tempLifoAlloc(), pattern))
         return false;
 
-    RegExpStatics *res = cx->global()->getRegExpStatics(cx);
-    if (!res)
-        return false;
-    RegExpObject *reobj = builder.build(escapedSourceStr, RegExpFlag(flags | res->getFlags()));
+    if (staticsUse == UseRegExpStatics) {
+        RegExpStatics* res = cx->global()->getRegExpStatics(cx);
+        if (!res)
+            return false;
+        flags = RegExpFlag(flags | res->getFlags());
+    }
+
+    /* Steps 11-15. */
+    RootedObject reobj(cx, builder.build(pattern, flags));
     if (!reobj)
         return false;
 
+    /* Step 16. */
+    result.set(reobj);
+    return true;
+}
+
+/*
+ * ES6 draft rc4 21.2.3.1 steps 5-10.
+ * ES6 draft rc4 B.2.5.1 steps 3-5.
+ * Compile a new |RegExpShared| for the |RegExpObject|.
+ */
+static bool
+CompileRegExpObject(JSContext* cx, RegExpObjectBuilder& builder, CallArgs args,
+                    RegExpCreationMode creationMode, bool patternIsRegExp=false)
+{
+    if (args.length() == 0) {
+        /*
+         * 21.2.3.1 step 10.
+         * B.2.5.1 step 5.
+         */
+        RegExpStatics* res = cx->global()->getRegExpStatics(cx);
+        if (!res)
+            return false;
+
+        RootedAtom empty(cx, cx->runtime()->emptyString);
+        RegExpObject* reobj = builder.build(empty, res->getFlags());
+        if (!reobj)
+            return false;
+
+        args.rval().setObject(*reobj);
+        return true;
+    }
+
+    RootedValue patternValue(cx, args.get(0));
+
+    /*
+     * 21.2.3.1 step 5
+     * B.2.5.1 step 3.
+     */
+    if (IsObjectWithClass(patternValue, ESClass_RegExp, cx)) {
+        /*
+         * B.2.5.1 step 3.a.
+         */
+        if (args.hasDefined(1) && creationMode == CreateForCompile) {
+            JS_ReportErrorNumber(cx, GetErrorMessage, nullptr, JSMSG_NEWREGEXP_FLAGGED);
+            return false;
+        }
+
+        /*
+         * Beware, patternObj may be a (transparent) proxy to a RegExp, so only
+         * use generic (proxyable) operations on patternObj that do not assume
+         * patternObj.is<RegExpObject>().
+         */
+        RootedObject patternObj(cx, &patternValue.toObject());
+
+        RootedAtom sourceAtom(cx);
+        RegExpFlag flags;
+        {
+            /*
+             * 21.2.3.1 step 5.a.
+             * B.2.5.1 step 3.a.
+             * Extract the 'source' from patternObj; do not reuse the
+             * RegExpShared since it may be from a different compartment.
+             */
+            RegExpGuard g(cx);
+            if (!RegExpToShared(cx, patternObj, &g))
+                return false;
+            sourceAtom = g->getSource();
+
+            if (args.hasDefined(1)) {
+                /* 21.2.3.1 step 5.c. */
+                flags = RegExpFlag(0);
+                RootedString flagStr(cx, ToString<CanGC>(cx, args[1]));
+                if (!flagStr)
+                    return false;
+                if (!ParseRegExpFlags(cx, flagStr, &flags))
+                    return false;
+            } else {
+                /*
+                 * 21.2.3.1 step 5.b.
+                 * B.2.5.1 step 3.c.
+                 */
+                flags = g->getFlags();
+            }
+        }
+
+        /*
+         * 21.2.3.1 steps 8-10.
+         * B.2.5.1 step 5.
+         */
+        RegExpObject* reobj = builder.build(sourceAtom, flags);
+        if (!reobj)
+            return false;
+
+        args.rval().setObject(*reobj);
+        return true;
+    }
+
+    RootedValue P(cx);
+    RootedValue F(cx);
+    /* 21.2.3.1 step 6. */
+    if (patternIsRegExp) {
+        MOZ_ASSERT(creationMode == CreateForConstruct);
+        RootedObject patternObj(cx, &patternValue.toObject());
+
+        /* 21.2.3.1 steps 6.a-b. */
+        if (!GetProperty(cx, patternObj, patternObj, cx->names().source, &P))
+            return false;
+
+        /* 21.2.3.1 step 6.c. */
+        if (!args.hasDefined(1)) {
+            /* 21.2.3.1 steps 6.c.i-ii. */
+            if (!GetProperty(cx, patternObj, patternObj, cx->names().flags, &F))
+                return false;
+        } else {
+            /* 21.2.3.1 steps 6.d. */
+            F = args[1];
+        }
+    } else {
+        /*
+         * 21.2.3.1 steps 7.a-b.
+         * B.2.5.1 steps 4.a-b.
+         */
+        P = patternValue;
+        F = args.get(1);
+    }
+
+    /*
+     * 21.2.3.1 steps 8-10.
+     * B.2.5.1 step 5.
+     */
+    RootedObject reobj(cx);
+    if (!RegExpInitialize(cx, builder, P, F, UseRegExpStatics, &reobj))
+        return false;
+
     args.rval().setObject(*reobj);
     return true;
 }
 
 MOZ_ALWAYS_INLINE bool
-IsRegExp(HandleValue v)
+IsRegExpObject(HandleValue v)
 {
     return v.isObject() && v.toObject().is<RegExpObject>();
 }
 
+/* ES6 draft rc3 7.2.8. */
+bool
+js::IsRegExp(JSContext* cx, HandleValue value, bool* result)
+{
+    /* Step 1. */
+    if (!value.isObject()) {
+        *result = false;
+        return true;
+    }
+    RootedObject obj(cx, &value.toObject());
+
+    /* Steps 2-3. */
+    RootedValue isRegExp(cx);
+    RootedId matchId(cx, SYMBOL_TO_JSID(cx->wellKnownSymbols().match));
+    if (!GetProperty(cx, obj, obj, matchId, &isRegExp))
+        return false;
+
+    /* Step 4. */
+    if (!isRegExp.isUndefined()) {
+        *result = ToBoolean(isRegExp);
+        return true;
+    }
+
+    /* Steps 5-6. */
+    *result = IsObjectWithClass(value, ESClass_RegExp, cx);
+    return true;
+}
+
+/* ES6 draft rc4 B.2.5.1. */
 MOZ_ALWAYS_INLINE bool
-regexp_compile_impl(JSContext *cx, CallArgs args)
+regexp_compile_impl(JSContext* cx, CallArgs args)
 {
-    MOZ_ASSERT(IsRegExp(args.thisv()));
+    MOZ_ASSERT(IsRegExpObject(args.thisv()));
+
+    /* Steps 3-5. */
     RegExpObjectBuilder builder(cx, &args.thisv().toObject().as<RegExpObject>());
-    return CompileRegExpObject(cx, builder, args);
+    return CompileRegExpObject(cx, builder, args, CreateForCompile);
 }
 
 static bool
-regexp_compile(JSContext *cx, unsigned argc, Value *vp)
+regexp_compile(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
-    return CallNonGenericMethod<IsRegExp, regexp_compile_impl>(cx, args);
+
+    /* Steps 1-2. */
+    return CallNonGenericMethod<IsRegExpObject, regexp_compile_impl>(cx, args);
 }
 
-static bool
-regexp_construct(JSContext *cx, unsigned argc, Value *vp)
+/* ES6 draft rc4 21.2.3.1. */
+bool
+js::regexp_construct(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
+    /* Steps 1-2. */
+    bool patternIsRegExp;
+    if (!IsRegExp(cx, args.get(0), &patternIsRegExp))
+        return false;
+
+    /* Step 4. */
     if (!args.isConstructing()) {
-        /*
-         * If first arg is regexp and no flags are given, just return the arg.
-         * Otherwise, delegate to the standard constructor.
-         * See ECMAv5 15.10.3.1.
-         */
-        if (args.hasDefined(0) &&
-            IsObjectWithClass(args[0], ESClass_RegExp, cx) &&
-            !args.hasDefined(1))
-        {
-            args.rval().set(args[0]);
-            return true;
+        /* Step 4.b. */
+        if (patternIsRegExp && !args.hasDefined(1)) {
+            RootedObject patternObj(cx, &args[0].toObject());
+
+            /* Steps 4.b.i-ii. */
+            RootedValue patternConstructor(cx);
+            if (!GetProperty(cx, patternObj, patternObj, cx->names().constructor, &patternConstructor))
+                return false;
+
+            /* Step 4.b.iii. */
+            if (patternConstructor.isObject() && patternConstructor.toObject() == args.callee()) {
+                args.rval().set(args[0]);
+                return true;
+            }
         }
     }
 
+    /* Steps 5-10. */
     RegExpObjectBuilder builder(cx);
-    return CompileRegExpObject(cx, builder, args);
+    return CompileRegExpObject(cx, builder, args, CreateForConstruct, patternIsRegExp);
 }
 
-MOZ_ALWAYS_INLINE bool
-regexp_toString_impl(JSContext *cx, CallArgs args)
+bool
+js::regexp_construct_no_statics(JSContext* cx, unsigned argc, Value* vp)
 {
-    MOZ_ASSERT(IsRegExp(args.thisv()));
+    CallArgs args = CallArgsFromVp(argc, vp);
 
-    JSString *str = args.thisv().toObject().as<RegExpObject>().toString(cx);
+    MOZ_ASSERT(args.length() == 1 || args.length() == 2);
+    MOZ_ASSERT(args[0].isString());
+    MOZ_ASSERT_IF(args.length() == 2, args[1].isString());
+    MOZ_ASSERT(!args.isConstructing());
+
+    /* Steps 1-6 are not required since pattern is always string. */
+
+    /* Steps 7-10. */
+    RegExpObjectBuilder builder(cx);
+    RootedObject reobj(cx);
+    if (!RegExpInitialize(cx, builder, args[0], args.get(1), DontUseRegExpStatics, &reobj))
+        return false;
+
+    args.rval().setObject(*reobj);
+    return true;
+}
+
+/* ES6 draft rev32 21.2.5.4. */
+MOZ_ALWAYS_INLINE bool
+regexp_global_impl(JSContext* cx, CallArgs args)
+{
+    MOZ_ASSERT(IsRegExpObject(args.thisv()));
+    Rooted<RegExpObject*> reObj(cx, &args.thisv().toObject().as<RegExpObject>());
+
+    /* Steps 4-6. */
+    args.rval().setBoolean(reObj->global());
+    return true;
+}
+
+static bool
+regexp_global(JSContext* cx, unsigned argc, JS::Value* vp)
+{
+    /* Steps 1-3. */
+    CallArgs args = CallArgsFromVp(argc, vp);
+    return CallNonGenericMethod<IsRegExpObject, regexp_global_impl>(cx, args);
+}
+
+/* ES6 draft rev32 21.2.5.5. */
+MOZ_ALWAYS_INLINE bool
+regexp_ignoreCase_impl(JSContext* cx, CallArgs args)
+{
+    MOZ_ASSERT(IsRegExpObject(args.thisv()));
+    Rooted<RegExpObject*> reObj(cx, &args.thisv().toObject().as<RegExpObject>());
+
+    /* Steps 4-6. */
+    args.rval().setBoolean(reObj->ignoreCase());
+    return true;
+}
+
+static bool
+regexp_ignoreCase(JSContext* cx, unsigned argc, JS::Value* vp)
+{
+    /* Steps 1-3. */
+    CallArgs args = CallArgsFromVp(argc, vp);
+    return CallNonGenericMethod<IsRegExpObject, regexp_ignoreCase_impl>(cx, args);
+}
+
+/* ES6 draft rev32 21.2.5.7. */
+MOZ_ALWAYS_INLINE bool
+regexp_multiline_impl(JSContext* cx, CallArgs args)
+{
+    MOZ_ASSERT(IsRegExpObject(args.thisv()));
+    Rooted<RegExpObject*> reObj(cx, &args.thisv().toObject().as<RegExpObject>());
+
+    /* Steps 4-6. */
+    args.rval().setBoolean(reObj->multiline());
+    return true;
+}
+
+static bool
+regexp_multiline(JSContext* cx, unsigned argc, JS::Value* vp)
+{
+    /* Steps 1-3. */
+    CallArgs args = CallArgsFromVp(argc, vp);
+    return CallNonGenericMethod<IsRegExpObject, regexp_multiline_impl>(cx, args);
+}
+
+/* ES6 draft rev32 21.2.5.10. */
+MOZ_ALWAYS_INLINE bool
+regexp_source_impl(JSContext* cx, CallArgs args)
+{
+    MOZ_ASSERT(IsRegExpObject(args.thisv()));
+    Rooted<RegExpObject*> reObj(cx, &args.thisv().toObject().as<RegExpObject>());
+
+    /* Step 5. */
+    RootedAtom src(cx, reObj->getSource());
+    if (!src)
+        return false;
+
+    /* Step 7. */
+    RootedString str(cx, EscapeRegExpPattern(cx, src));
     if (!str)
         return false;
 
@@ -365,83 +517,48 @@ regexp_toString_impl(JSContext *cx, CallArgs args)
 }
 
 static bool
-regexp_toString(JSContext *cx, unsigned argc, Value *vp)
+regexp_source(JSContext* cx, unsigned argc, JS::Value* vp)
 {
+    /* Steps 1-4. */
     CallArgs args = CallArgsFromVp(argc, vp);
-    return CallNonGenericMethod<IsRegExp, regexp_toString_impl>(cx, args);
+    return CallNonGenericMethod<IsRegExpObject, regexp_source_impl>(cx, args);
 }
 
-/* ES6 draft rev29 21.2.5.3 RegExp.prototype.flags */
-bool
-regexp_flags(JSContext *cx, unsigned argc, JS::Value *vp)
+/* ES6 draft rev32 21.2.5.12. */
+MOZ_ALWAYS_INLINE bool
+regexp_sticky_impl(JSContext* cx, CallArgs args)
 {
-    CallArgs args = CallArgsFromVp(argc, vp);
-
-    /* Steps 1-2. */
-    if (!args.thisv().isObject()) {
-        char *bytes = DecompileValueGenerator(cx, JSDVG_SEARCH_STACK, args.thisv(), NullPtr());
-        if (!bytes)
-            return false;
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_UNEXPECTED_TYPE,
-                             bytes, "not an object");
-        js_free(bytes);
-        return false;
-    }
-    RootedObject thisObj(cx, &args.thisv().toObject());
-
-    /* Step 3. */
-    StringBuffer sb(cx);
+    MOZ_ASSERT(IsRegExpObject(args.thisv()));
+    Rooted<RegExpObject*> reObj(cx, &args.thisv().toObject().as<RegExpObject>());
 
     /* Steps 4-6. */
-    RootedValue global(cx);
-    if (!JSObject::getProperty(cx, thisObj, thisObj, cx->names().global, &global))
-        return false;
-    if (ToBoolean(global) && !sb.append('g'))
-        return false;
-
-    /* Steps 7-9. */
-    RootedValue ignoreCase(cx);
-    if (!JSObject::getProperty(cx, thisObj, thisObj, cx->names().ignoreCase, &ignoreCase))
-        return false;
-    if (ToBoolean(ignoreCase) && !sb.append('i'))
-        return false;
-
-    /* Steps 10-12. */
-    RootedValue multiline(cx);
-    if (!JSObject::getProperty(cx, thisObj, thisObj, cx->names().multiline, &multiline))
-        return false;
-    if (ToBoolean(multiline) && !sb.append('m'))
-        return false;
-
-    /* Steps 13-15. */
-    RootedValue unicode(cx);
-    if (!JSObject::getProperty(cx, thisObj, thisObj, cx->names().unicode, &unicode))
-        return false;
-    if (ToBoolean(unicode) && !sb.append('u'))
-        return false;
-
-    /* Steps 16-18. */
-    RootedValue sticky(cx);
-    if (!JSObject::getProperty(cx, thisObj, thisObj, cx->names().sticky, &sticky))
-        return false;
-    if (ToBoolean(sticky) && !sb.append('y'))
-        return false;
-
-    /* Step 19. */
-    args.rval().setString(sb.finishString());
+    args.rval().setBoolean(reObj->sticky());
     return true;
 }
 
-static const JSPropertySpec regexp_properties[] = {
-    JS_PSG("flags", regexp_flags, 0),
+static bool
+regexp_sticky(JSContext* cx, unsigned argc, JS::Value* vp)
+{
+    /* Steps 1-3. */
+    CallArgs args = CallArgsFromVp(argc, vp);
+    return CallNonGenericMethod<IsRegExpObject, regexp_sticky_impl>(cx, args);
+}
+
+const JSPropertySpec js::regexp_properties[] = {
+    JS_SELF_HOSTED_GET("flags", "RegExpFlagsGetter", 0),
+    JS_PSG("global", regexp_global, 0),
+    JS_PSG("ignoreCase", regexp_ignoreCase, 0),
+    JS_PSG("multiline", regexp_multiline, 0),
+    JS_PSG("source", regexp_source, 0),
+    JS_PSG("sticky", regexp_sticky, 0),
     JS_PS_END
 };
 
-static const JSFunctionSpec regexp_methods[] = {
+const JSFunctionSpec js::regexp_methods[] = {
 #if JS_HAS_TOSOURCE
-    JS_FN(js_toSource_str,  regexp_toString,    0,0),
+    JS_SELF_HOSTED_FN(js_toSource_str, "RegExpToString", 0, 0),
 #endif
-    JS_FN(js_toString_str,  regexp_toString,    0,0),
+    JS_SELF_HOSTED_FN(js_toString_str, "RegExpToString", 0, 0),
     JS_FN("compile",        regexp_compile,     2,0),
     JS_FN("exec",           regexp_exec,        1,0),
     JS_FN("test",           regexp_test,        1,0),
@@ -470,10 +587,10 @@ static const JSFunctionSpec regexp_methods[] = {
 
 #define DEFINE_STATIC_GETTER(name, code)                                        \
     static bool                                                                 \
-    name(JSContext *cx, unsigned argc, Value *vp)                               \
+    name(JSContext* cx, unsigned argc, Value* vp)                               \
     {                                                                           \
         CallArgs args = CallArgsFromVp(argc, vp);                               \
-        RegExpStatics *res = cx->global()->getRegExpStatics(cx);                \
+        RegExpStatics* res = cx->global()->getRegExpStatics(cx);                \
         if (!res)                                                               \
             return false;                                                       \
         code;                                                                   \
@@ -499,9 +616,9 @@ DEFINE_STATIC_GETTER(static_paren9_getter,       STATIC_PAREN_GETTER_CODE(9))
 
 #define DEFINE_STATIC_SETTER(name, code)                                        \
     static bool                                                                 \
-    name(JSContext *cx, unsigned argc, Value *vp)                               \
+    name(JSContext* cx, unsigned argc, Value* vp)                               \
     {                                                                           \
-        RegExpStatics *res = cx->global()->getRegExpStatics(cx);                \
+        RegExpStatics* res = cx->global()->getRegExpStatics(cx);                \
         if (!res)                                                               \
             return false;                                                       \
         code;                                                                   \
@@ -509,10 +626,10 @@ DEFINE_STATIC_GETTER(static_paren9_getter,       STATIC_PAREN_GETTER_CODE(9))
     }
 
 static bool
-static_input_setter(JSContext *cx, unsigned argc, Value *vp)
+static_input_setter(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
-    RegExpStatics *res = cx->global()->getRegExpStatics(cx);
+    RegExpStatics* res = cx->global()->getRegExpStatics(cx);
     if (!res)
         return false;
 
@@ -526,10 +643,10 @@ static_input_setter(JSContext *cx, unsigned argc, Value *vp)
 }
 
 static bool
-static_multiline_setter(JSContext *cx, unsigned argc, Value *vp)
+static_multiline_setter(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
-    RegExpStatics *res = cx->global()->getRegExpStatics(cx);
+    RegExpStatics* res = cx->global()->getRegExpStatics(cx);
     if (!res)
         return false;
 
@@ -539,7 +656,7 @@ static_multiline_setter(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-static const JSPropertySpec regexp_static_props[] = {
+const JSPropertySpec js::regexp_static_props[] = {
     JS_PSGS("input", static_input_getter, static_input_setter,
             JSPROP_PERMANENT | JSPROP_ENUMERATE),
     JS_PSGS("multiline", static_multiline_getter, static_multiline_setter,
@@ -566,14 +683,12 @@ static const JSPropertySpec regexp_static_props[] = {
     JS_PS_END
 };
 
-JSObject *
-js_InitRegExpClass(JSContext *cx, HandleObject obj)
+JSObject*
+js::CreateRegExpPrototype(JSContext* cx, JSProtoKey key)
 {
-    MOZ_ASSERT(obj->isNative());
+    MOZ_ASSERT(key == JSProto_RegExp);
 
-    Rooted<GlobalObject*> global(cx, &obj->as<GlobalObject>());
-
-    Rooted<RegExpObject*> proto(cx, global->createBlankPrototype<RegExpObject>(cx));
+    Rooted<RegExpObject*> proto(cx, cx->global()->createBlankPrototype<RegExpObject>(cx));
     if (!proto)
         return nullptr;
     proto->NativeObject::setPrivate(nullptr);
@@ -583,39 +698,45 @@ js_InitRegExpClass(JSContext *cx, HandleObject obj)
     if (!builder.build(empty, RegExpFlag(0)))
         return nullptr;
 
-    if (!DefinePropertiesAndFunctions(cx, proto, regexp_properties, regexp_methods))
-        return nullptr;
-
-    RootedFunction ctor(cx);
-    ctor = global->createConstructor(cx, regexp_construct, cx->names().RegExp, 2);
-    if (!ctor)
-        return nullptr;
-
-    if (!LinkConstructorAndPrototype(cx, ctor, proto))
-        return nullptr;
-
-    /* Add static properties to the RegExp constructor. */
-    if (!JS_DefineProperties(cx, ctor, regexp_static_props))
-        return nullptr;
-
-    if (!GlobalObject::initBuiltinConstructor(cx, global, JSProto_RegExp, ctor, proto))
-        return nullptr;
-
     return proto;
 }
 
-RegExpRunStatus
-js::ExecuteRegExp(JSContext *cx, HandleObject regexp, HandleString string,
-                  MatchPairs *matches, RegExpStaticsUpdate staticsUpdate)
+static bool
+ReportLastIndexNonwritable(JSContext* cx)
 {
-    /* Step 1 (b) was performed by CallNonGenericMethod. */
+    JS_ReportErrorNumber(cx, GetErrorMessage, nullptr, JSMSG_READ_ONLY, "\"lastIndex\"");
+    return false;
+}
+
+static bool
+SetLastIndex(JSContext* cx, Handle<RegExpObject*> reobj, double lastIndex)
+{
+    if (!reobj->lookup(cx, cx->names().lastIndex)->writable())
+        return ReportLastIndexNonwritable(cx);
+
+    reobj->setLastIndex(lastIndex);
+    return true;
+}
+
+/* ES6 final draft 21.2.5.2.2. */
+RegExpRunStatus
+js::ExecuteRegExp(JSContext* cx, HandleObject regexp, HandleString string,
+                  MatchPairs* matches, RegExpStaticsUpdate staticsUpdate)
+{
+    /*
+     * WARNING: Despite the presence of spec step comment numbers, this
+     *          algorithm isn't consistent with any ES6 version, draft or
+     *          otherwise.  YOU HAVE BEEN WARNED.
+     */
+
+    /* Steps 1-2 performed by the caller. */
     Rooted<RegExpObject*> reobj(cx, &regexp->as<RegExpObject>());
 
     RegExpGuard re(cx);
     if (!reobj->getShared(cx, &re))
         return RegExpRunStatus_Error;
 
-    RegExpStatics *res;
+    RegExpStatics* res;
     if (staticsUpdate == UpdateRegExpStatics) {
         res = cx->global()->getRegExpStatics(cx);
         if (!res)
@@ -624,16 +745,15 @@ js::ExecuteRegExp(JSContext *cx, HandleObject regexp, HandleString string,
         res = nullptr;
     }
 
-    /* Step 3. */
     RootedLinearString input(cx, string->ensureLinear(cx));
     if (!input)
         return RegExpRunStatus_Error;
 
-    /* Step 4. */
-    RootedValue lastIndex(cx, reobj->getLastIndex());
+    /* Step 3. */
     size_t length = input->length();
 
-    /* Step 5. */
+    /* Steps 4-5. */
+    RootedValue lastIndex(cx, reobj->getLastIndex());
     int searchIndex;
     if (lastIndex.isInt32()) {
         /* Aggressively avoid doubles. */
@@ -643,9 +763,13 @@ js::ExecuteRegExp(JSContext *cx, HandleObject regexp, HandleString string,
         if (!ToInteger(cx, lastIndex, &d))
             return RegExpRunStatus_Error;
 
-        /* Inlined steps 6, 7, 9a with doubles to detect failure case. */
+        /* Inlined steps 6-10, 15.a with doubles to detect failure case. */
         if (reobj->needUpdateLastIndex() && (d < 0 || d > length)) {
-            reobj->zeroLastIndex();
+            /* Steps 15.a.i-ii. */
+            if (!SetLastIndex(cx, reobj, 0))
+                return RegExpRunStatus_Error;
+
+            /* Step 15.a.iii. */
             return RegExpRunStatus_Success_NotFound;
         }
 
@@ -653,7 +777,7 @@ js::ExecuteRegExp(JSContext *cx, HandleObject regexp, HandleString string,
     }
 
     /*
-     * Steps 6-7 (with sticky extension).
+     * Steps 6-10.
      *
      * Also make sure that we have a MatchPairs for regexps which update their
      * last index, as we won't compute the last index otherwise.
@@ -666,29 +790,37 @@ js::ExecuteRegExp(JSContext *cx, HandleObject regexp, HandleString string,
         matches = &alternateMatches.ref();
     }
 
-    /* Step 9a. */
+    /* Step 15.a. */
     if (searchIndex < 0 || size_t(searchIndex) > length) {
-        reobj->zeroLastIndex();
+        /* Steps 15.a.i-ii. */
+        if (!SetLastIndex(cx, reobj, 0))
+            return RegExpRunStatus_Error;
+
+        /* Step 15.a.iii. */
         return RegExpRunStatus_Success_NotFound;
     }
 
-    /* Steps 8-21. */
+    /* Step 14-29. */
     RegExpRunStatus status = ExecuteRegExpImpl(cx, res, *re, input, searchIndex, matches);
     if (status == RegExpRunStatus_Error)
         return RegExpRunStatus_Error;
 
-    /* Steps 9a and 11 (with sticky extension). */
-    if (status == RegExpRunStatus_Success_NotFound)
-        reobj->zeroLastIndex();
-    else if (reobj->needUpdateLastIndex())
-        reobj->setLastIndex((*matches)[0].limit);
+    if (status == RegExpRunStatus_Success_NotFound) {
+        /* Steps 15.a.i-ii. */
+        if (!SetLastIndex(cx, reobj, 0))
+            return RegExpRunStatus_Error;
+    } else if (reobj->needUpdateLastIndex()) {
+        /* Steps 18.a-b. */
+        if (!SetLastIndex(cx, reobj, (*matches)[0].limit))
+            return RegExpRunStatus_Error;
+    }
 
     return status;
 }
 
 /* ES5 15.10.6.2 (and 15.10.6.3, which calls 15.10.6.2). */
 static RegExpRunStatus
-ExecuteRegExp(JSContext *cx, CallArgs args, MatchPairs *matches)
+ExecuteRegExp(JSContext* cx, CallArgs args, MatchPairs* matches)
 {
     /* Step 1 (a) was performed by CallNonGenericMethod. */
     RootedObject regexp(cx, &args.thisv().toObject());
@@ -703,7 +835,7 @@ ExecuteRegExp(JSContext *cx, CallArgs args, MatchPairs *matches)
 
 /* ES5 15.10.6.2. */
 static bool
-regexp_exec_impl(JSContext *cx, HandleObject regexp, HandleString string,
+regexp_exec_impl(JSContext* cx, HandleObject regexp, HandleString string,
                  RegExpStaticsUpdate staticsUpdate, MutableHandleValue rval)
 {
     /* Execute regular expression and gather matches. */
@@ -722,7 +854,7 @@ regexp_exec_impl(JSContext *cx, HandleObject regexp, HandleString string,
 }
 
 static bool
-regexp_exec_impl(JSContext *cx, CallArgs args)
+regexp_exec_impl(JSContext* cx, CallArgs args)
 {
     RootedObject regexp(cx, &args.thisv().toObject());
     RootedString string(cx, ToString<CanGC>(cx, args.get(0)));
@@ -733,16 +865,16 @@ regexp_exec_impl(JSContext *cx, CallArgs args)
 }
 
 bool
-js::regexp_exec(JSContext *cx, unsigned argc, Value *vp)
+js::regexp_exec(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
-    return CallNonGenericMethod(cx, IsRegExp, regexp_exec_impl, args);
+    return CallNonGenericMethod(cx, IsRegExpObject, regexp_exec_impl, args);
 }
 
 /* Separate interface for use by IonMonkey. */
 bool
-js::regexp_exec_raw(JSContext *cx, HandleObject regexp, HandleString input,
-                    MatchPairs *maybeMatches, MutableHandleValue output)
+js::regexp_exec_raw(JSContext* cx, HandleObject regexp, HandleString input,
+                    MatchPairs* maybeMatches, MutableHandleValue output)
 {
     // The MatchPairs will always be passed in, but RegExp execution was
     // successful only if the pairs have actually been filled in.
@@ -752,11 +884,11 @@ js::regexp_exec_raw(JSContext *cx, HandleObject regexp, HandleString input,
 }
 
 bool
-js::regexp_exec_no_statics(JSContext *cx, unsigned argc, Value *vp)
+js::regexp_exec_no_statics(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
     MOZ_ASSERT(args.length() == 2);
-    MOZ_ASSERT(IsRegExp(args[0]));
+    MOZ_ASSERT(IsRegExpObject(args[0]));
     MOZ_ASSERT(args[1].isString());
 
     RootedObject regexp(cx, &args[0].toObject());
@@ -767,7 +899,7 @@ js::regexp_exec_no_statics(JSContext *cx, unsigned argc, Value *vp)
 
 /* ES5 15.10.6.3. */
 static bool
-regexp_test_impl(JSContext *cx, CallArgs args)
+regexp_test_impl(JSContext* cx, CallArgs args)
 {
     RegExpRunStatus status = ExecuteRegExp(cx, args, nullptr);
     args.rval().setBoolean(status == RegExpRunStatus_Success);
@@ -776,7 +908,7 @@ regexp_test_impl(JSContext *cx, CallArgs args)
 
 /* Separate interface for use by IonMonkey. */
 bool
-js::regexp_test_raw(JSContext *cx, HandleObject regexp, HandleString input, bool *result)
+js::regexp_test_raw(JSContext* cx, HandleObject regexp, HandleString input, bool* result)
 {
     RegExpRunStatus status = ExecuteRegExp(cx, regexp, input, nullptr, UpdateRegExpStatics);
     *result = (status == RegExpRunStatus_Success);
@@ -784,18 +916,18 @@ js::regexp_test_raw(JSContext *cx, HandleObject regexp, HandleString input, bool
 }
 
 bool
-js::regexp_test(JSContext *cx, unsigned argc, Value *vp)
+js::regexp_test(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
-    return CallNonGenericMethod(cx, IsRegExp, regexp_test_impl, args);
+    return CallNonGenericMethod(cx, IsRegExpObject, regexp_test_impl, args);
 }
 
 bool
-js::regexp_test_no_statics(JSContext *cx, unsigned argc, Value *vp)
+js::regexp_test_no_statics(JSContext* cx, unsigned argc, Value* vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
     MOZ_ASSERT(args.length() == 2);
-    MOZ_ASSERT(IsRegExp(args[0]));
+    MOZ_ASSERT(IsRegExpObject(args[0]));
     MOZ_ASSERT(args[1].isString());
 
     RootedObject regexp(cx, &args[0].toObject());
