@@ -6,14 +6,16 @@
 #include "nsJARInputStream.h"
 #include "nsJAR.h"
 #include "nsIFile.h"
+#include "nsIX509Cert.h"
 #include "nsIConsoleService.h"
 #include "nsICryptoHash.h"
+#include "nsIDataSignatureVerifier.h"
 #include "prprf.h"
 #include "mozilla/Omnijar.h"
 
 #ifdef XP_UNIX
   #include <sys/stat.h>
-#elif defined (XP_WIN) || defined(XP_OS2)
+#elif defined (XP_WIN)
   #include <io.h>
 #endif
 
@@ -23,8 +25,8 @@ using namespace mozilla;
 // nsJARManifestItem declaration
 //----------------------------------------------
 /*
- * nsJARManifestItem contains meta-information pertaining 
- * to an individual JAR entry, taken from the 
+ * nsJARManifestItem contains meta-information pertaining
+ * to an individual JAR entry, taken from the
  * META-INF/MANIFEST.MF and META-INF/ *.SF files.
  * This is security-critical information, defined here so it is not
  * accessible from anywhere else.
@@ -41,7 +43,7 @@ class nsJARManifestItem
 public:
   JARManifestItemType mType;
 
-  // True if the second step of verification (VerifyEntry) 
+  // True if the second step of verification (VerifyEntry)
   // has taken place:
   bool                entryVerified;
 
@@ -72,21 +74,14 @@ nsJARManifestItem::~nsJARManifestItem()
 //----------------------------------------------
 // nsJAR constructor/destructor
 //----------------------------------------------
-static bool
-DeleteManifestEntry(nsHashKey* aKey, void* aData, void* closure)
-{
-//-- deletes an entry in  mManifestData.
-  delete (nsJARManifestItem*)aData;
-  return true;
-}
 
 // The following initialization makes a guess of 10 entries per jarfile.
 nsJAR::nsJAR(): mZip(new nsZipArchive()),
-                mManifestData(nullptr, nullptr, DeleteManifestEntry, nullptr, 10),
+                mManifestData(8),
                 mParsedManifest(false),
                 mGlobalStatus(JAR_MANIFEST_NOT_PARSED),
-                mReleaseTime(PR_INTERVAL_NO_TIMEOUT), 
-                mCache(nullptr), 
+                mReleaseTime(PR_INTERVAL_NO_TIMEOUT),
+                mCache(nullptr),
                 mLock("nsJAR::mLock"),
                 mTotalItemsInManifest(0),
                 mOpened(false)
@@ -98,22 +93,22 @@ nsJAR::~nsJAR()
   Close();
 }
 
-NS_IMPL_QUERY_INTERFACE1(nsJAR, nsIZipReader)
+NS_IMPL_QUERY_INTERFACE(nsJAR, nsIZipReader)
 NS_IMPL_ADDREF(nsJAR)
 
 // Custom Release method works with nsZipReaderCache...
-nsrefcnt nsJAR::Release(void) 
+MozExternalRefCountType nsJAR::Release(void)
 {
-  nsrefcnt count; 
-  NS_PRECONDITION(0 != mRefCnt, "dup release"); 
+  nsrefcnt count;
+  NS_PRECONDITION(0 != mRefCnt, "dup release");
   count = --mRefCnt;
-  NS_LOG_RELEASE(this, count, "nsJAR"); 
+  NS_LOG_RELEASE(this, count, "nsJAR");
   if (0 == count) {
-    mRefCnt = 1; /* stabilize */ 
-    /* enable this to find non-threadsafe destructors: */ 
-    /* NS_ASSERT_OWNINGTHREAD(nsJAR); */ 
+    mRefCnt = 1; /* stabilize */
+    /* enable this to find non-threadsafe destructors: */
+    /* NS_ASSERT_OWNINGTHREAD(nsJAR); */
     delete this;
-    return 0; 
+    return 0;
   }
   else if (1 == count && mCache) {
 #ifdef DEBUG
@@ -122,8 +117,8 @@ nsrefcnt nsJAR::Release(void)
       mCache->ReleaseZip(this);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to release zip file");
   }
-  return count; 
-} 
+  return count;
+}
 
 //----------------------------------------------
 // nsIZipReader implementation
@@ -138,7 +133,7 @@ nsJAR::Open(nsIFile* zipFile)
   mZipFile = zipFile;
   mOuterZipEntry.Truncate();
   mOpened = true;
-  
+
   // The omnijar is special, it is opened early on and closed late
   // this avoids reopening it
   nsRefPtr<nsZipArchive> zip = mozilla::Omnijar::GetReader(zipFile);
@@ -146,7 +141,7 @@ nsJAR::Open(nsIFile* zipFile)
     mZip = zip;
     return NS_OK;
   }
-  return mZip->OpenArchive(zipFile);
+  return mZip->OpenArchive(zipFile, mCache ? mCache->IsMustCacheFdEnabled() : false);
 }
 
 NS_IMETHODIMP
@@ -176,6 +171,23 @@ nsJAR::OpenInner(nsIZipReader *aZipReader, const nsACString &aZipEntry)
   return mZip->OpenArchive(handle);
 }
 
+NS_IMETHODIMP
+nsJAR::OpenMemory(void* aData, uint32_t aLength)
+{
+  NS_ENSURE_ARG_POINTER(aData);
+  if (mOpened) return NS_ERROR_FAILURE; // Already open!
+
+  mOpened = true;
+
+  nsRefPtr<nsZipHandle> handle;
+  nsresult rv = nsZipHandle::Init(static_cast<uint8_t*>(aData), aLength,
+                                  getter_AddRefs(handle));
+  if (NS_FAILED(rv))
+    return rv;
+
+  return mZip->OpenArchive(handle);
+}
+
 NS_IMETHODIMP
 nsJAR::GetFile(nsIFile* *result)
 {
@@ -189,7 +201,7 @@ nsJAR::Close()
 {
   mOpened = false;
   mParsedManifest = false;
-  mManifestData.Reset();
+  mManifestData.Clear();
   mGlobalStatus = JAR_MANIFEST_NOT_PARSED;
   mTotalItemsInManifest = 0;
 
@@ -257,14 +269,13 @@ nsJAR::Extract(const nsACString &aEntryName, nsIFile* outFile)
   return NS_OK;
 }
 
-NS_IMETHODIMP    
+NS_IMETHODIMP
 nsJAR::GetEntry(const nsACString &aEntryName, nsIZipEntry* *result)
 {
   nsZipItem* zipItem = mZip->GetItem(PromiseFlatCString(aEntryName).get());
   NS_ENSURE_TRUE(zipItem, NS_ERROR_FILE_TARGET_DOES_NOT_EXIST);
 
   nsJARItem* jarItem = new nsJARItem(zipItem);
-  NS_ENSURE_TRUE(jarItem, NS_ERROR_OUT_OF_MEMORY);
 
   NS_ADDREF(*result = jarItem);
   return NS_OK;
@@ -287,10 +298,6 @@ nsJAR::FindEntries(const nsACString &aPattern, nsIUTF8StringEnumerator **result)
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsIUTF8StringEnumerator *zipEnum = new nsJAREnumerator(find);
-  if (!zipEnum) {
-    delete find;
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
 
   NS_ADDREF(*result = zipEnum);
   return NS_OK;
@@ -303,7 +310,7 @@ nsJAR::GetInputStream(const nsACString &aFilename, nsIInputStream** result)
 }
 
 NS_IMETHODIMP
-nsJAR::GetInputStreamWithSpec(const nsACString& aJarDirSpec, 
+nsJAR::GetInputStreamWithSpec(const nsACString& aJarDirSpec,
                           const nsACString &aEntryName, nsIInputStream** result)
 {
   NS_ENSURE_ARG_POINTER(result);
@@ -318,7 +325,6 @@ nsJAR::GetInputStreamWithSpec(const nsACString& aJarDirSpec,
   }
   nsJARInputStream* jis = new nsJARInputStream();
   // addref now so we can call InitFile/InitDirectory()
-  NS_ENSURE_TRUE(jis, NS_ERROR_OUT_OF_MEMORY);
   NS_ADDREF(*result = jis);
 
   nsresult rv = NS_OK;
@@ -334,12 +340,13 @@ nsJAR::GetInputStreamWithSpec(const nsACString& aJarDirSpec,
 }
 
 NS_IMETHODIMP
-nsJAR::GetCertificatePrincipal(const nsACString &aFilename, nsICertificatePrincipal** aPrincipal)
+nsJAR::GetSigningCert(const nsACString& aFilename, nsIX509Cert** aSigningCert)
 {
   //-- Parameter check
-  if (!aPrincipal)
+  if (!aSigningCert) {
     return NS_ERROR_NULL_POINTER;
-  *aPrincipal = nullptr;
+  }
+  *aSigningCert = nullptr;
 
   // Don't check signatures in the omnijar - this is only
   // interesting for extensions/XPIs.
@@ -359,8 +366,7 @@ nsJAR::GetCertificatePrincipal(const nsACString &aFilename, nsICertificatePrinci
   if (!aFilename.IsEmpty())
   {
     //-- Find the item
-    nsCStringKey key(aFilename);
-    nsJARManifestItem* manItem = static_cast<nsJARManifestItem*>(mManifestData.Get(&key));
+    nsJARManifestItem* manItem = mManifestData.Get(aFilename);
     if (!manItem)
       return NS_OK;
     //-- Verify the item against the manifest
@@ -378,17 +384,16 @@ nsJAR::GetCertificatePrincipal(const nsACString &aFilename, nsICertificatePrinci
   else // User wants identity of signer w/o verifying any entries
     requestedStatus = mGlobalStatus;
 
-  if (requestedStatus != JAR_VALID_MANIFEST)
+  if (requestedStatus != JAR_VALID_MANIFEST) {
     ReportError(aFilename, requestedStatus);
-  else // Valid signature
-  {
-    *aPrincipal = mPrincipal;
-    NS_IF_ADDREF(*aPrincipal);
+  } else { // Valid signature
+    *aSigningCert = mSigningCert;
+    NS_IF_ADDREF(*aSigningCert);
   }
   return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsJAR::GetManifestEntriesCount(uint32_t* count)
 {
   *count = mTotalItemsInManifest;
@@ -403,10 +408,30 @@ nsJAR::GetJarPath(nsACString& aResult)
   return mZipFile->GetNativePath(aResult);
 }
 
+nsresult
+nsJAR::GetNSPRFileDesc(PRFileDesc** aNSPRFileDesc)
+{
+  if (!aNSPRFileDesc) {
+    return NS_ERROR_ILLEGAL_VALUE;
+  }
+  *aNSPRFileDesc = nullptr;
+
+  if (!mZip) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsRefPtr<nsZipHandle> handle = mZip->GetFD();
+  if (!handle) {
+    return NS_ERROR_FAILURE;
+  }
+
+  return handle->GetNSPRFileDesc(aNSPRFileDesc);
+}
+
 //----------------------------------------------
 // nsJAR private implementation
 //----------------------------------------------
-nsresult 
+nsresult
 nsJAR::LoadEntry(const nsACString &aFilename, char** aBuf, uint32_t* aBufLen)
 {
   //-- Get a stream for reading the file
@@ -414,7 +439,7 @@ nsJAR::LoadEntry(const nsACString &aFilename, char** aBuf, uint32_t* aBufLen)
   nsCOMPtr<nsIInputStream> manifestStream;
   rv = GetInputStream(aFilename, getter_AddRefs(manifestStream));
   if (NS_FAILED(rv)) return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
-  
+
   //-- Read the manifest file into memory
   char* buf;
   uint64_t len64;
@@ -426,7 +451,7 @@ nsJAR::LoadEntry(const nsACString &aFilename, char** aBuf, uint32_t* aBufLen)
   if (!buf) return NS_ERROR_OUT_OF_MEMORY;
   uint32_t bytesRead;
   rv = manifestStream->Read(buf, len, &bytesRead);
-  if (bytesRead != len) 
+  if (bytesRead != len)
     rv = NS_ERROR_FILE_CORRUPTED;
   if (NS_FAILED(rv)) {
     free(buf);
@@ -443,6 +468,10 @@ nsJAR::LoadEntry(const nsACString &aFilename, char** aBuf, uint32_t* aBufLen)
 int32_t
 nsJAR::ReadLine(const char** src)
 {
+  if (!*src) {
+    return 0;
+  }
+
   //--Moves pointer to beginning of next line and returns line length
   //  not including CR/LF.
   int32_t length;
@@ -540,7 +569,7 @@ nsJAR::ParseManifest()
 
   rv = LoadEntry(manifestFilename, getter_Copies(manifestBuffer), &manifestLen);
   if (NS_FAILED(rv)) return rv;
-  
+
   //-- Get its corresponding signature file
   nsAutoCString sigFilename(manifestFilename);
   int32_t extension = sigFilename.RFindChar('.') + 1;
@@ -565,8 +594,8 @@ nsJAR::ParseManifest()
   }
 
   //-- Get the signature verifier service
-  nsCOMPtr<nsISignatureVerifier> verifier = 
-           do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
+  nsCOMPtr<nsIDataSignatureVerifier> verifier(
+    do_GetService("@mozilla.org/security/datasignatureverifier;1", &rv));
   if (NS_FAILED(rv)) // No signature verifier available
   {
     mGlobalStatus = JAR_NO_MANIFEST;
@@ -576,20 +605,21 @@ nsJAR::ParseManifest()
 
   //-- Verify that the signature file is a valid signature of the SF file
   int32_t verifyError;
-  rv = verifier->VerifySignature(sigBuffer, sigLen, manifestBuffer, manifestLen, 
-                                 &verifyError, getter_AddRefs(mPrincipal));
+  rv = verifier->VerifySignature(sigBuffer, sigLen, manifestBuffer, manifestLen,
+                                 &verifyError, getter_AddRefs(mSigningCert));
   if (NS_FAILED(rv)) return rv;
-  if (mPrincipal && verifyError == 0)
+  if (mSigningCert && verifyError == nsIDataSignatureVerifier::VERIFY_OK) {
     mGlobalStatus = JAR_VALID_MANIFEST;
-  else if (verifyError == nsISignatureVerifier::VERIFY_ERROR_UNKNOWN_CA)
+  } else if (verifyError == nsIDataSignatureVerifier::VERIFY_ERROR_UNKNOWN_ISSUER) {
     mGlobalStatus = JAR_INVALID_UNKNOWN_CA;
-  else
+  } else {
     mGlobalStatus = JAR_INVALID_SIG;
+  }
 
   //-- Parse the SF file. If the verification above failed, principal
   // is null, and ParseOneFile will mark the relevant entries as invalid.
-  // if ParseOneFile fails, then it has no effect, and we can safely 
-  // continue to the next SF file, or return. 
+  // if ParseOneFile fails, then it has no effect, and we can safely
+  // continue to the next SF file, or return.
   ParseOneFile(manifestBuffer, JAR_SF);
   mParsedManifest = true;
 
@@ -621,9 +651,9 @@ nsJAR::ParseOneFile(const char* filebuf, int16_t aFileType)
 
   nsJARManifestItem* curItemMF = nullptr;
   bool foundName = false;
-  if (aFileType == JAR_MF)
-    if (!(curItemMF = new nsJARManifestItem()))
-      return NS_ERROR_OUT_OF_MEMORY;
+  if (aFileType == JAR_MF) {
+    curItemMF = new nsJARManifestItem();
+  }
 
   nsAutoCString curItemName;
   nsAutoCString storedSectionDigest;
@@ -633,20 +663,20 @@ nsJAR::ParseOneFile(const char* filebuf, int16_t aFileType)
     curPos = nextLineStart;
     linelen = ReadLine(&nextLineStart);
     curLine.Assign(curPos, linelen);
-    if (linelen == 0) 
+    if (linelen == 0)
     // end of section (blank line or end-of-file)
     {
       if (aFileType == JAR_MF)
       {
         mTotalItemsInManifest++;
         if (curItemMF->mType != JAR_INVALID)
-        { 
+        {
           //-- Did this section have a name: line?
           if(!foundName)
             curItemMF->mType = JAR_INVALID;
-          else 
+          else
           {
-            //-- If it's an internal item, it must correspond 
+            //-- If it's an internal item, it must correspond
             //   to a valid jar entry
             if (curItemMF->mType == JAR_INTERNAL)
             {
@@ -656,9 +686,9 @@ nsJAR::ParseOneFile(const char* filebuf, int16_t aFileType)
                 curItemMF->mType = JAR_INVALID;
             }
             //-- Check for duplicates
-            nsCStringKey key(curItemName);
-            if (mManifestData.Exists(&key))
+            if (mManifestData.Contains(curItemName)) {
               curItemMF->mType = JAR_INVALID;
+            }
           }
         }
 
@@ -670,25 +700,21 @@ nsJAR::ParseOneFile(const char* filebuf, int16_t aFileType)
           CalculateDigest(sectionStart, sectionLength,
                           curItemMF->calculatedSectionDigest);
           //-- Save item in the hashtable
-          nsCStringKey itemKey(curItemName);
-          mManifestData.Put(&itemKey, (void*)curItemMF);
+          mManifestData.Put(curItemName, curItemMF);
         }
         if (nextLineStart == nullptr) // end-of-file
           break;
 
         sectionStart = nextLineStart;
-        if (!(curItemMF = new nsJARManifestItem()))
-          return NS_ERROR_OUT_OF_MEMORY;
+        curItemMF = new nsJARManifestItem();
       } // (aFileType == JAR_MF)
       else
-        //-- file type is SF, compare digest with calculated 
+        //-- file type is SF, compare digest with calculated
         //   section digests from MF file.
       {
         if (foundName)
         {
-          nsJARManifestItem* curItemSF;
-          nsCStringKey key(curItemName);
-          curItemSF = (nsJARManifestItem*)mManifestData.Get(&key);
+          nsJARManifestItem* curItemSF = mManifestData.Get(curItemName);
           if(curItemSF)
           {
             NS_ASSERTION(curItemSF->status == JAR_NOT_SIGNED,
@@ -740,7 +766,7 @@ nsJAR::ParseOneFile(const char* filebuf, int16_t aFileType)
     //-- Lines to look for:
     // (1) Digest:
     if (lineName.LowerCaseEqualsLiteral("sha1-digest"))
-    //-- This is a digest line, save the data in the appropriate place 
+    //-- This is a digest line, save the data in the appropriate place
     {
       if(aFileType == JAR_MF)
         curItemMF->storedEntryDigest = lineData;
@@ -748,7 +774,7 @@ nsJAR::ParseOneFile(const char* filebuf, int16_t aFileType)
         storedSectionDigest = lineData;
       continue;
     }
-    
+
     // (2) Name: associates this manifest section with a file in the jar.
     if (!foundName && lineName.LowerCaseEqualsLiteral("name"))
     {
@@ -757,7 +783,7 @@ nsJAR::ParseOneFile(const char* filebuf, int16_t aFileType)
       continue;
     }
 
-    // (3) Magic: this may be an inline Javascript. 
+    // (3) Magic: this may be an inline Javascript.
     //     We can't do any other kind of magic.
     if (aFileType == JAR_MF && lineName.LowerCaseEqualsLiteral("magic"))
     {
@@ -798,7 +824,7 @@ nsJAR::VerifyEntry(nsJARManifestItem* aManItem, const char* aEntryData,
 void nsJAR::ReportError(const nsACString &aFilename, int16_t errorCode)
 {
   //-- Generate error message
-  nsAutoString message; 
+  nsAutoString message;
   message.AssignLiteral("Signature Verification Error: the signature on ");
   if (!aFilename.IsEmpty())
     AppendASCIItoUTF16(aFilename, message);
@@ -828,7 +854,7 @@ void nsJAR::ReportError(const nsACString &aFilename, int16_t errorCode)
   default:
     message.AppendLiteral("of an unknown problem.");
   }
-  
+
   // Report error in JS console
   nsCOMPtr<nsIConsoleService> console(do_GetService("@mozilla.org/consoleservice;1"));
   if (console)
@@ -839,7 +865,7 @@ void nsJAR::ReportError(const nsACString &aFilename, int16_t errorCode)
   char* messageCstr = ToNewCString(message);
   if (!messageCstr) return;
   fprintf(stderr, "%s\n", messageCstr);
-  nsMemory::Free(messageCstr);
+  free(messageCstr);
 #endif
 }
 
@@ -861,8 +887,8 @@ nsresult nsJAR::CalculateDigest(const char* aInBuf, uint32_t aLen,
   return hasher->Finish(true, digest);
 }
 
-NS_IMPL_ISUPPORTS1(nsJAREnumerator, nsIUTF8StringEnumerator)
-  
+NS_IMPL_ISUPPORTS(nsJAREnumerator, nsIUTF8StringEnumerator)
+
 //----------------------------------------------
 // nsJAREnumerator::HasMore
 //----------------------------------------------
@@ -903,7 +929,7 @@ nsJAREnumerator::GetNext(nsACString& aResult)
 }
 
 
-NS_IMPL_ISUPPORTS1(nsJARItem, nsIZipEntry)
+NS_IMPL_ISUPPORTS(nsJARItem, nsIZipEntry)
 
 nsJARItem::nsJARItem(nsZipItem* aZipItem)
     : mSize(aZipItem->Size()),
@@ -1016,11 +1042,12 @@ nsJARItem::GetPermissions(uint32_t* aPermissions)
 ////////////////////////////////////////////////////////////////////////////////
 // nsIZipReaderCache
 
-NS_IMPL_ISUPPORTS3(nsZipReaderCache, nsIZipReaderCache, nsIObserver, nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS(nsZipReaderCache, nsIZipReaderCache, nsIObserver, nsISupportsWeakReference)
 
 nsZipReaderCache::nsZipReaderCache()
   : mLock("nsZipReaderCache.mLock")
-  , mZips(16)
+  , mZips()
+  , mMustCacheFd(false)
 #ifdef ZIP_CACHE_HIT_RATE
     ,
     mZipCacheLookups(0),
@@ -1034,10 +1061,10 @@ nsZipReaderCache::nsZipReaderCache()
 NS_IMETHODIMP
 nsZipReaderCache::Init(uint32_t cacheSize)
 {
-  mCacheSize = cacheSize; 
-  
-// Register as a memory pressure observer 
-  nsCOMPtr<nsIObserverService> os = 
+  mCacheSize = cacheSize;
+
+// Register as a memory pressure observer
+  nsCOMPtr<nsIObserverService> os =
            do_GetService("@mozilla.org/observer-service;1");
   if (os)
   {
@@ -1050,22 +1077,21 @@ nsZipReaderCache::Init(uint32_t cacheSize)
   return NS_OK;
 }
 
-static bool
-DropZipReaderCache(nsHashKey *aKey, void *aData, void* closure)
+static PLDHashOperator
+DropZipReaderCache(const nsACString &aKey, nsJAR* aZip, void*)
 {
-  nsJAR* zip = (nsJAR*)aData;
-  zip->SetZipReaderCache(nullptr);
-  return true;
+  aZip->SetZipReaderCache(nullptr);
+  return PL_DHASH_NEXT;
 }
 
 nsZipReaderCache::~nsZipReaderCache()
 {
-  mZips.Enumerate(DropZipReaderCache, nullptr);
+  mZips.EnumerateRead(DropZipReaderCache, nullptr);
 
 #ifdef ZIP_CACHE_HIT_RATE
   printf("nsZipReaderCache size=%d hits=%d lookups=%d rate=%f%% flushes=%d missed %d\n",
-         mCacheSize, mZipCacheHits, mZipCacheLookups, 
-         (float)mZipCacheHits / mZipCacheLookups, 
+         mCacheSize, mZipCacheHits, mZipCacheLookups,
+         (float)mZipCacheHits / mZipCacheLookups,
          mZipCacheFlushes, mZipSyncMisses);
 #endif
 }
@@ -1085,9 +1111,7 @@ nsZipReaderCache::IsCached(nsIFile* zipFile, bool* aResult)
 
   uri.Insert(NS_LITERAL_CSTRING("file:"), 0);
 
-  nsCStringKey key(uri);
-
-  *aResult = mZips.Exists(&key);
+  *aResult = mZips.Contains(uri);
   return NS_OK;
 }
 
@@ -1109,34 +1133,25 @@ nsZipReaderCache::GetZip(nsIFile* zipFile, nsIZipReader* *result)
 
   uri.Insert(NS_LITERAL_CSTRING("file:"), 0);
 
-  nsCStringKey key(uri);
-  nsJAR* zip = static_cast<nsJAR*>(static_cast<nsIZipReader*>(mZips.Get(&key))); // AddRefs
+  nsRefPtr<nsJAR> zip;
+  mZips.Get(uri, getter_AddRefs(zip));
   if (zip) {
 #ifdef ZIP_CACHE_HIT_RATE
     mZipCacheHits++;
 #endif
     zip->ClearReleaseTime();
-  }
-  else {
+  } else {
     zip = new nsJAR();
-    if (zip == nullptr)
-        return NS_ERROR_OUT_OF_MEMORY;
-    NS_ADDREF(zip);
     zip->SetZipReaderCache(this);
-
     rv = zip->Open(zipFile);
     if (NS_FAILED(rv)) {
-      NS_RELEASE(zip);
       return rv;
     }
 
-#ifdef DEBUG
-    bool collision =
-#endif
-      mZips.Put(&key, static_cast<nsIZipReader*>(zip)); // AddRefs to 2
-    NS_ASSERTION(!collision, "horked");
+    MOZ_ASSERT(!mZips.Contains(uri));
+    mZips.Put(uri, zip);
   }
-  *result = zip;
+  zip.forget(result);
   return rv;
 }
 
@@ -1150,6 +1165,8 @@ nsZipReaderCache::GetInnerZip(nsIFile* zipFile, const nsACString &entry,
   nsresult rv = GetZip(zipFile, getter_AddRefs(outerZipReader));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  MutexAutoLock lock(mLock);
+
 #ifdef ZIP_CACHE_HIT_RATE
   mZipCacheLookups++;
 #endif
@@ -1162,62 +1179,140 @@ nsZipReaderCache::GetInnerZip(nsIFile* zipFile, const nsACString &entry,
   uri.AppendLiteral("!/");
   uri.Append(entry);
 
-  nsCStringKey key(uri);
-  nsJAR* zip = static_cast<nsJAR*>(static_cast<nsIZipReader*>(mZips.Get(&key))); // AddRefs
+  nsRefPtr<nsJAR> zip;
+  mZips.Get(uri, getter_AddRefs(zip));
   if (zip) {
 #ifdef ZIP_CACHE_HIT_RATE
     mZipCacheHits++;
 #endif
     zip->ClearReleaseTime();
-  }
-  else {
+  } else {
     zip = new nsJAR();
-    NS_ADDREF(zip);
     zip->SetZipReaderCache(this);
 
     rv = zip->OpenInner(outerZipReader, entry);
     if (NS_FAILED(rv)) {
-      NS_RELEASE(zip);
       return rv;
     }
-#ifdef DEBUG
-    bool collision =
-#endif
-    mZips.Put(&key, static_cast<nsIZipReader*>(zip)); // AddRefs to 2
-    NS_ASSERTION(!collision, "horked");
+
+    MOZ_ASSERT(!mZips.Contains(uri));
+    mZips.Put(uri, zip);
   }
-  *result = zip;
+  zip.forget(result);
   return rv;
 }
 
-static bool
-FindOldestZip(nsHashKey *aKey, void *aData, void* closure)
+NS_IMETHODIMP
+nsZipReaderCache::SetMustCacheFd(nsIFile* zipFile, bool aMustCacheFd)
 {
-  nsJAR** oldestPtr = (nsJAR**)closure;
+#if defined(XP_WIN)
+  MOZ_CRASH("Not implemented");
+  return NS_ERROR_NOT_IMPLEMENTED;
+#else
+
+  if (!aMustCacheFd) {
+    return NS_OK;
+  }
+
+  if (!zipFile) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsresult rv;
+  nsAutoCString uri;
+  rv = zipFile->GetNativePath(uri);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  uri.Insert(NS_LITERAL_CSTRING("file:"), 0);
+
+  MutexAutoLock lock(mLock);
+
+  mMustCacheFd = aMustCacheFd;
+
+  nsRefPtr<nsJAR> zip;
+  mZips.Get(uri, getter_AddRefs(zip));
+  if (!zip) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // Flush the file from the cache if its file descriptor was not cached.
+  PRFileDesc* fd = nullptr;
+  zip->GetNSPRFileDesc(&fd);
+  if (!fd) {
+#ifdef ZIP_CACHE_HIT_RATE
+    mZipCacheFlushes++;
+#endif
+    zip->SetZipReaderCache(nullptr);
+    mZips.Remove(uri);
+  }
+  return NS_OK;
+#endif /* XP_WIN */
+}
+
+NS_IMETHODIMP
+nsZipReaderCache::GetFd(nsIFile* zipFile, PRFileDesc** aRetVal)
+{
+#if defined(XP_WIN)
+  MOZ_CRASH("Not implemented");
+  return NS_ERROR_NOT_IMPLEMENTED;
+#else
+  if (!zipFile) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsresult rv;
+  nsAutoCString uri;
+  rv = zipFile->GetNativePath(uri);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  uri.Insert(NS_LITERAL_CSTRING("file:"), 0);
+
+  MutexAutoLock lock(mLock);
+  nsRefPtr<nsJAR> zip;
+  mZips.Get(uri, getter_AddRefs(zip));
+  if (!zip) {
+    return NS_ERROR_FAILURE;
+  }
+
+  zip->ClearReleaseTime();
+  rv = zip->GetNSPRFileDesc(aRetVal);
+  // Do this to avoid possible deadlock on mLock with ReleaseZip().
+  MutexAutoUnlock unlock(mLock);
+  nsRefPtr<nsJAR> zipTemp = zip.forget();
+  return rv;
+#endif /* XP_WIN */
+}
+
+static PLDHashOperator
+FindOldestZip(const nsACString &aKey, nsJAR* aZip, void* aClosure)
+{
+  nsJAR** oldestPtr = static_cast<nsJAR**>(aClosure);
   nsJAR* oldest = *oldestPtr;
-  nsJAR* current = (nsJAR*)aData;
+  nsJAR* current = aZip;
   PRIntervalTime currentReleaseTime = current->GetReleaseTime();
   if (currentReleaseTime != PR_INTERVAL_NO_TIMEOUT) {
     if (oldest == nullptr ||
         currentReleaseTime < oldest->GetReleaseTime()) {
       *oldestPtr = current;
-    }    
+    }
   }
-  return true;
+  return PL_DHASH_NEXT;
 }
 
-struct ZipFindData {nsJAR* zip; bool found;}; 
+struct ZipFindData {nsJAR* zip; bool found;};
 
-static bool
-FindZip(nsHashKey *aKey, void *aData, void* closure)
+static PLDHashOperator
+FindZip(const nsACString &aKey, nsJAR* aZip, void* aClosure)
 {
-  ZipFindData* find_data = (ZipFindData*)closure;
+  ZipFindData* find_data = static_cast<ZipFindData*>(aClosure);
 
-  if (find_data->zip == (nsJAR*)aData) {
-    find_data->found = true; 
-    return false;
+  if (find_data->zip == aZip) {
+    find_data->found = true;
+    return PL_DHASH_STOP;
   }
-  return true;
+  return PL_DHASH_NEXT;
 }
 
 nsresult
@@ -1226,23 +1321,23 @@ nsZipReaderCache::ReleaseZip(nsJAR* zip)
   nsresult rv;
   MutexAutoLock lock(mLock);
 
-  // It is possible that two thread compete for this zip. The dangerous 
+  // It is possible that two thread compete for this zip. The dangerous
   // case is where one thread Releases the zip and discovers that the ref
   // count has gone to one. Before it can call this ReleaseZip method
   // another thread calls our GetZip method. The ref count goes to two. That
   // second thread then Releases the zip and the ref count goes to one. It
   // then tries to enter this ReleaseZip method and blocks while the first
-  // thread is still here. The first thread continues and remove the zip from 
+  // thread is still here. The first thread continues and remove the zip from
   // the cache and calls its Release method sending the ref count to 0 and
   // deleting the zip. However, the second thread is still blocked at the
   // start of ReleaseZip, but the 'zip' param now hold a reference to a
   // deleted zip!
-  // 
+  //
   // So, we are going to try safeguarding here by searching our hashtable while
-  // locked here for the zip. We return fast if it is not found. 
+  // locked here for the zip. We return fast if it is not found.
 
   ZipFindData find_data = {zip, false};
-  mZips.Enumerate(FindZip, &find_data);
+  mZips.EnumerateRead(FindZip, &find_data);
   if (!find_data.found) {
 #ifdef ZIP_CACHE_HIT_RATE
     mZipSyncMisses++;
@@ -1256,10 +1351,10 @@ nsZipReaderCache::ReleaseZip(nsJAR* zip)
     return NS_OK;
 
   nsJAR* oldest = nullptr;
-  mZips.Enumerate(FindOldestZip, &oldest);
-  
+  mZips.EnumerateRead(FindOldestZip, &oldest);
+
   // Because of the craziness above it is possible that there is no zip that
-  // needs removing. 
+  // needs removing.
   if (!oldest)
     return NS_OK;
 
@@ -1281,9 +1376,11 @@ nsZipReaderCache::ReleaseZip(nsJAR* zip)
     uri.Append(oldest->mOuterZipEntry);
   }
 
-  nsCStringKey key(uri);
+  // Retrieving and removing the JAR must be done without an extra AddRef
+  // and Release, or we'll trigger nsJAR::Release's magic refcount 1 case
+  // an extra time and trigger a deadlock.
   nsRefPtr<nsJAR> removed;
-  mZips.Remove(&key, (nsISupports **)removed.StartAssignment());
+  mZips.Remove(uri, getter_AddRefs(removed));
   NS_ASSERTION(removed, "botched");
   NS_ASSERTION(oldest == removed, "removed wrong entry");
 
@@ -1293,46 +1390,29 @@ nsZipReaderCache::ReleaseZip(nsJAR* zip)
   return NS_OK;
 }
 
-static bool
-FindFlushableZip(nsHashKey *aKey, void *aData, void* closure)
+static PLDHashOperator
+FindFlushableZip(const nsACString &aKey, nsRefPtr<nsJAR>& aCurrent, void*)
 {
-  nsHashKey** flushableKeyPtr = (nsHashKey**)closure;
-  nsJAR* current = (nsJAR*)aData;
-  
-  if (current->GetReleaseTime() != PR_INTERVAL_NO_TIMEOUT) {
-    *flushableKeyPtr = aKey;
-    current->SetZipReaderCache(nullptr);
-    return false;
+  if (aCurrent->GetReleaseTime() != PR_INTERVAL_NO_TIMEOUT) {
+    aCurrent->SetZipReaderCache(nullptr);
+    return PL_DHASH_REMOVE;
   }
-  return true;
+  return PL_DHASH_NEXT;
 }
 
 NS_IMETHODIMP
 nsZipReaderCache::Observe(nsISupports *aSubject,
-                          const char *aTopic, 
-                          const PRUnichar *aSomeData)
+                          const char *aTopic,
+                          const char16_t *aSomeData)
 {
   if (strcmp(aTopic, "memory-pressure") == 0) {
     MutexAutoLock lock(mLock);
-    while (true) {
-      nsHashKey* flushable = nullptr;
-      mZips.Enumerate(FindFlushableZip, &flushable); 
-      if ( ! flushable )
-        break;
-#ifdef DEBUG
-      bool removed =
-#endif
-        mZips.Remove(flushable);   // Releases
-      NS_ASSERTION(removed, "botched");
-
-#ifdef xDEBUG_jband
-      printf("flushed something from the jar cache\n");
-#endif
-    }
+    mZips.Enumerate(FindFlushableZip, nullptr);
   }
   else if (strcmp(aTopic, "chrome-flush-caches") == 0) {
-    mZips.Enumerate(DropZipReaderCache, nullptr);
-    mZips.Reset();
+    MutexAutoLock lock(mLock);
+    mZips.EnumerateRead(DropZipReaderCache, nullptr);
+    mZips.Clear();
   }
   else if (strcmp(aTopic, "flush-cache-entry") == 0) {
     nsCOMPtr<nsIFile> file = do_QueryInterface(aSubject);
@@ -1344,10 +1424,11 @@ nsZipReaderCache::Observe(nsISupports *aSubject,
       return NS_OK;
 
     uri.Insert(NS_LITERAL_CSTRING("file:"), 0);
-    nsCStringKey key(uri);
 
-    MutexAutoLock lock(mLock);    
-    nsJAR* zip = static_cast<nsJAR*>(static_cast<nsIZipReader*>(mZips.Get(&key)));
+    MutexAutoLock lock(mLock);
+
+    nsRefPtr<nsJAR> zip;
+    mZips.Get(uri, getter_AddRefs(zip));
     if (!zip)
       return NS_OK;
 
@@ -1357,8 +1438,7 @@ nsZipReaderCache::Observe(nsISupports *aSubject,
 
     zip->SetZipReaderCache(nullptr);
 
-    mZips.Remove(&key);
-    NS_RELEASE(zip);
+    mZips.Remove(uri);
   }
   return NS_OK;
 }
