@@ -16,9 +16,9 @@ class nsIOutputStream;
 
 namespace mozilla { namespace net {
 
-class nsHttpPipeline MOZ_FINAL : public nsAHttpConnection
-                               , public nsAHttpTransaction
-                               , public nsAHttpSegmentReader
+class nsHttpPipeline final : public nsAHttpConnection
+                           , public nsAHttpTransaction
+                           , public nsAHttpSegmentReader
 {
 public:
     NS_DECL_THREADSAFE_ISUPPORTS
@@ -28,7 +28,7 @@ public:
 
     nsHttpPipeline();
 
-  bool ResponseTimeoutEnabled() const MOZ_OVERRIDE MOZ_FINAL {
+  bool ResponseTimeoutEnabled() const override final {
     return true;
   }
 
@@ -57,7 +57,7 @@ private:
     }
 
     // overload of nsAHttpTransaction::QueryPipeline()
-    nsHttpPipeline *QueryPipeline() MOZ_OVERRIDE;
+    nsHttpPipeline *QueryPipeline() override;
 
     nsRefPtr<nsAHttpConnection>   mConnection;
     nsTArray<nsAHttpTransaction*> mRequestQ;  // array of transactions
@@ -94,9 +94,9 @@ private:
     uint32_t  mHttp1xTransactionCount;
 
     // For support of OnTransportStatus()
-    uint64_t  mReceivingFromProgress;
-    uint64_t  mSendingToProgress;
-    bool      mSuppressSendEvents;
+    int64_t  mReceivingFromProgress;
+    int64_t  mSendingToProgress;
+    bool     mSuppressSendEvents;
 };
 
 }} // namespace mozilla::net
